@@ -1,10 +1,10 @@
-from typing import Tuple
+from typing import List, Tuple
 import pandas as pd
 from ..wrapper.mysql import RawDatabaseConnector
 from ..view.raw_models import *
 
 
-class RawDataApi(object):
+class RawDataApi:
     def get_raw_cm_index_price_df(self, start_date, end_date):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
@@ -65,12 +65,16 @@ class RawDataApi(object):
             except Exception as e:
                 print('Failed to get data <err_msg> {}'.format(e))
 
-    def get_wind_fund_info(self):
+    def get_wind_fund_info(self, funds: Tuple[str] = ()):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
                 query = db_session.query(
                     WindFundInfo
                 )
+                if funds:
+                    query = query.filter(
+                        WindFundInfo.wind_id.in_(funds),
+                    )
                 df = pd.read_sql(query.statement, query.session.bind)
                 return df
 
@@ -196,17 +200,21 @@ class RawDataApi(object):
                 print('Failed to get data <err_msg> {}'.format(e))
                 return None
 
-    def get_em_tradedates(self, start_date, end_date):
+    def get_em_tradedates(self, start_date='', end_date=''):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
                 query = db_session.query(
                     EmTradeDates
-                ).filter(
-                    EmTradeDates.TRADEDATES >= start_date,
-                    EmTradeDates.TRADEDATES <= end_date
                 )
-                df = pd.read_sql(query.statement, query.session.bind)
-                return df
+                if start_date:
+                    query = query.filter(
+                        EmTradeDates.TRADEDATES >= start_date,
+                    )
+                if end_date:
+                    query = query.filter(
+                        EmTradeDates.TRADEDATES <= end_date,
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
                 print('Failed to get data <err_msg> {}'.format(e))
                 return None
@@ -310,13 +318,17 @@ class RawDataApi(object):
                 print('Failed to get data <err_msg> {}'.format(e))
                 return None
 
-    def get_em_index_price(self, start_date, end_date, index_id_list):
+    def get_em_index_price(self, start_date, end_date, index_id_list: Tuple = ()):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
                 query = db_session.query(
                     EmIndexPrice
-                ).filter(
-                    EmIndexPrice.em_id.in_(index_id_list),
+                )
+                if index_id_list:
+                    query = query.filter(
+                        EmIndexPrice.em_id.in_(index_id_list),
+                    )
+                query = query.filter(
                     EmIndexPrice.datetime >= start_date,
                     EmIndexPrice.datetime <= end_date,
                 )
@@ -454,7 +466,7 @@ class RawDataApi(object):
                 print(f'Failed to get data <err_msg> {e} from {EmStockDailyInfo.__tablename__}')
                 return None
 
-    def get_em_stock_fin_fac(self, stock_list: Tuple[str] = (), date_list: Tuple[str] = ()):
+    def get_em_stock_fin_fac(self, *, stock_list: Tuple[str] = (), date_list: Tuple[str] = (), start_date: str = '', end_date: str = ''):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
                 query = db_session.query(
@@ -468,11 +480,57 @@ class RawDataApi(object):
                     query = query.filter(
                         EmStockFinFac.DATES.in_(date_list)
                     )
+                else:
+                    if start_date:
+                        query = query.filter(
+                            EmStockFinFac.DATES >= start_date,
+                        )
+                    if end_date:
+                        query = query.filter(
+                            EmStockFinFac.DATES <= end_date,
+                        )
                 return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
                 print(f'Failed to get data <err_msg> {e} from {EmStockFinFac.__tablename__}')
                 return None
 
+    def get_em_stock_estimate_fac(self, predict_year: int, start_date: str = '', end_date: str = ''):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    EmStockEstimateFac
+                )
+                if start_date:
+                    query = query.filter(
+                        EmStockEstimateFac.DATES >= start_date,
+                    )
+                if end_date:
+                    query = query.filter(
+                        EmStockEstimateFac.DATES <= end_date,
+                    )
+                query = query.filter(
+                    EmStockEstimateFac.predict_year == predict_year,
+                )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get data <err_msg> {e} from {EmStockEstimateFac.__tablename__}')
+                return None
+
+    def get_em_index_info(self, index_list: Tuple[str] = ()):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    EmIndexInfo
+                )
+                if index_list:
+                    query = query.filter(
+                        EmIndexInfo.em_id.in_(index_list)
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get data <err_msg> {e} from {EmIndexInfo.__tablename__}')
+                return None
+
     def get_em_index_component(self, start_date: str, end_date: str, index_list: Tuple[str] = ()):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
@@ -492,6 +550,21 @@ class RawDataApi(object):
                 print(f'Failed to get data <err_msg> {e} from {EmIndexComponent.__tablename__}')
                 return None
 
+    def get_cs_index_component(self, index_list: Tuple[str] = ()):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    CSIndexComponent
+                )
+                if index_list:
+                    query = query.filter(
+                        CSIndexComponent.index_id.in_(index_list)
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get data <err_msg> {e} from {CSIndexComponent.__tablename__}')
+                return None
+
     def get_em_fund_holding_rate(self, start_date: str):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
@@ -505,3 +578,151 @@ class RawDataApi(object):
             except Exception as e:
                 print('Failed to get data <err_msg> {}'.format(e))
                 return None
+
+    def get_em_fund_list(self, date: str, limit = -1):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    EmFundList.datetime,
+                    EmFundList.all_live_fund_list,
+                    EmFundList.delisted_fund_list,
+                ).filter(
+                    EmFundList.datetime <= date,
+                )
+                if limit != -1:
+                    query = query.order_by(EmFundList.datetime.desc()).limit(limit)
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+                return None
+
+    def delete_em_fund_info(self, funds: List[str]):
+        with RawDatabaseConnector().managed_session() as db_session:
+            query = db_session.query(
+                EmFundInfo
+            ).filter(
+                EmFundInfo.CODES.in_(funds),
+            ).delete(synchronize_session=False)
+            db_session.commit()
+
+    def get_em_fund_info(self, funds: List[str]):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    EmFundInfo
+                ).filter(
+                    EmFundInfo.CODES.in_(funds),
+                )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+                return None
+
+    def delete_em_fund_fee(self, funds: List[str]):
+        with RawDatabaseConnector().managed_session() as db_session:
+            query = db_session.query(
+                EmFundFee
+            ).filter(
+                EmFundFee.CODES.in_(funds),
+            ).delete(synchronize_session=False)
+            db_session.commit()
+
+    def get_em_fund_fee(self, funds: List[str]):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    EmFundFee
+                ).filter(
+                    EmFundFee.CODES.in_(funds),
+                )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+                return None
+
+    def get_em_fund_benchmark(self, end_date: str):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    EmFundBenchmark
+                ).filter(
+                    EmFundBenchmark.DATES <= end_date,
+                )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+                return None
+
+    def get_wind_holder_structure(self, start_date: str, wind_fund_list:list):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    WindFundHolderStructure
+                ).filter(
+                    WindFundHolderStructure.END_DT >= start_date,
+                    WindFundHolderStructure.S_INFO_WINDCODE.in_(wind_fund_list),
+                )
+                df = pd.read_sql(query.statement, query.session.bind)
+                return df
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+                return None
+
+    def get_wind_fund_stock_portfolio(self, start_date: str, wind_fund_list:list):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    WindFundStockPortfolio
+                ).filter(
+                    WindFundStockPortfolio.F_PRT_ENDDATE >= start_date,
+                    WindFundStockPortfolio.S_INFO_WINDCODE.in_(wind_fund_list),
+                )
+                df = pd.read_sql(query.statement, query.session.bind)
+                return df
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+                return None
+
+    def get_wind_fund_nav(self, start_date: str, wind_fund_list:list):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    WindFundNav
+                ).filter(
+                    WindFundNav.PRICE_DATE >= start_date,
+                    WindFundNav.F_INFO_WINDCODE.in_(wind_fund_list),
+                )
+                df = pd.read_sql(query.statement, query.session.bind)
+                return df
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+                return None
+
+    def get_wind_manager_info(self, wind_fund_list:list):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    WindFundManager
+                ).filter(
+                    WindFundManager.F_INFO_WINDCODE.in_(wind_fund_list),
+                )
+                df = pd.read_sql(query.statement, query.session.bind)
+                return df
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+                return None
+
+    def get_wind_indus_portfolio(self, start_date: str, wind_fund_list:list):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    WindIndPortfolio
+                ).filter(
+                    WindIndPortfolio.F_PRT_ENDDATE >= start_date,
+                    WindIndPortfolio.S_INFO_WINDCODE.in_(wind_fund_list),
+                )
+                df = pd.read_sql(query.statement, query.session.bind)
+                return df
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+                return None
\ No newline at end of file
