@@ -1,14 +1,19 @@
 
-import pandas as pd
-import traceback
 import datetime
+import traceback
+import json
+import numpy as np
+import pandas as pd
+from typing import List, Dict
+
+from ....constant import SectorType, INDEX_PRICE_EXTRA_TRADE_DAYS
 from ...api.raw import RawDataApi
 from ...api.basic import BasicDataApi
 from ...view.basic_models import *
 from .basic_data_helper import BasicDataHelper
 
 
-class BasicDataPart1(object):
+class BasicDataPart1:
 
     def __init__(self, data_helper: BasicDataHelper):
         self._data_helper = data_helper
@@ -91,6 +96,46 @@ class BasicDataPart1(object):
         df = pd.read_csv('./data/index_info.csv')
         self._data_helper._upload_basic(df, IndexInfo.__table__.name)
 
+    @staticmethod
+    def _lambda_func_index_component_2(em_id_to_index_id: Dict[str, str], x: str):
+        info = json.loads(x)
+        r = []
+        for one in info:
+            em_id = one[0]
+            if em_id in ['000915.SH', '000916.SH']:
+                em_id = em_id.split('.')[0] + '.CSI'
+            r.append((em_id_to_index_id[em_id], one[1]))
+        return json.dumps(r)
+
+    @staticmethod
+    def _lambda_func_index_component_3(wind_id_to_id: Dict[str, str], x: str):
+        if x is None:
+            return x
+        r = []
+        info = json.loads(x)
+        for one in info:
+            try:
+                r.append(wind_id_to_id[one + '.OF'])
+            except KeyError:
+                r.append(one + '!0')
+        return json.dumps(r)
+
+    def index_component(self):
+        api = BasicDataApi()
+        index_info_df = api.get_index_info()
+        index_info_df = index_info_df[index_info_df.em_id.notna()]
+        index_info_df = index_info_df.set_index('em_id')
+        em_id_to_index_id: Dict[str, str] = index_info_df['index_id'].to_dict()
+
+        df = RawDataApi().get_cs_index_component().drop(columns=['_update_time'])
+        df['index_id'] = df['index_id'].map(em_id_to_index_id)
+        df['sector'] = df['sector'].transform(lambda x: BasicDataPart1._lambda_func_index_component_2(em_id_to_index_id, x))
+        df['id_cat'] = df['id_cat'].transform(lambda x: x.name)
+        fund_info_df = api.get_fund_info().set_index('wind_id')
+        wind_id_to_id: Dict[str, str] = fund_info_df['fund_id'].to_dict()
+        df['related_funds'] = df['related_funds'].transform(lambda x: BasicDataPart1._lambda_func_index_component_3(wind_id_to_id, x))
+        self._data_helper._upload_basic(df, IndexComponent.__table__.name)
+
     def fund_nav_from_rq(self, start_date, end_date):
         df = RawDataApi().get_rq_fund_nav(start_date, end_date)
         df['fund_id'] = df.apply(
@@ -118,73 +163,85 @@ class BasicDataPart1(object):
                     'UNITYIELD10K': 'daily_profit',
                     'YIELDOF7DAYS': 'weekly_yield'
                 })
-            df['change_rate'] = df.adjusted_net_value / df.adjusted_net_value.shift(1) - 1
             df['weekly_yield'] = df['weekly_yield'].map(lambda x: x / 100.0)
+
+            anv_df = df.pivot_table(index='datetime', columns='fund_id', values='adjusted_net_value').fillna(method='ffill')
+            df = df.set_index(['datetime', 'fund_id'])
+            df['change_rate'] = (anv_df / anv_df.shift(1) - 1).stack()
+
             end_date = datetime.datetime.strptime(end_date, '%Y%m%d').date()
             BasicDataApi().delete_fund_nav(del_date, end_date)
-            df = df[df.datetime >= del_date.date()]
-            self._data_helper._upload_basic(df, FundNav.__table__.name)
+            df = df[df.index.get_level_values(level='datetime').date >= del_date.date()]
+            self._data_helper._upload_basic(df.reset_index(), FundNav.__table__.name)
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
+    def fund_benchmark(self):
+        df = pd.read_csv('./data/benchmark_res2.csv')
+        df = df[['fund_id', 'fund_em_id', 'benchmark', 'index_text',   'assets', 'industry']].rename(columns={'fund_em_id': 'em_id'})
+        self._data_helper._upload_basic(df, FundBenchmark.__table__.name)
+
     def index_price(self, start_date, end_date):
         try:
-            start_date = datetime.datetime.strptime(start_date, '%Y%m%d') - datetime.timedelta(days=3)
+            # 多取N个交易日的数据
+            start_date = datetime.datetime.strptime(start_date, '%Y%m%d') - datetime.timedelta(days=90)
             start_date = start_date.strftime('%Y%m%d')
-            cm_index_price = RawDataApi().get_raw_cm_index_price_df(start_date, end_date)
-            cxindex_index_price = RawDataApi().get_cxindex_index_price_df(start_date, end_date)
-            yahoo_index_price = RawDataApi().get_yahoo_index_price_df(start_date, end_date)
-            rq_index_price = RawDataApi().get_rq_index_price_df(start_date, end_date)
+            trading_day_df = BasicDataApi().get_trading_day_list(start_date, end_date)
+            assert trading_day_df.datetime.size >= INDEX_PRICE_EXTRA_TRADE_DAYS, f'should at least have {INDEX_PRICE_EXTRA_TRADE_DAYS} trading days'
+            # del_date在start_date后边一天，确保可以正确计算ret等daily difference
+            start_date = trading_day_df.datetime.array[-INDEX_PRICE_EXTRA_TRADE_DAYS]
+            del_date = trading_day_df.datetime.array[-(INDEX_PRICE_EXTRA_TRADE_DAYS-1)]
+            print(f'[index_price] (start_date){start_date} (del_date){del_date} (end_date){end_date}')
+            # 准备多个数据源
+            api = RawDataApi()
+            cm_index_price = api.get_raw_cm_index_price_df(start_date, end_date).drop(columns=['_update_time']).set_index('datetime').fillna(method='ffill')
+            cxindex_index_price = api.get_cxindex_index_price_df(start_date, end_date).drop(columns=['_update_time'])
+            yahoo_index_price = api.get_yahoo_index_price_df(start_date, end_date).drop(columns=['_update_time'])
+            em_id_list: Dict[str, str] = self._index_info_df[['em_id', 'index_id']].dropna().set_index('em_id').to_dict()['index_id']
+            em_index_price = api.get_em_index_price(start_date, end_date, em_id_list.keys()).drop(columns=['_update_time'])
 
             df_list = []
+            # 处理Choice数据
+            index_id_list = []
+            for em_id, index_id in em_id_list.items():
+                df = em_index_price[em_index_price['em_id'] == em_id]
+                df = df.drop(columns=['em_id'])
+                df['index_id'] = index_id
+                df['ret'] = df.close / df.close.shift(1) - 1
+                df = df[df.datetime >= pd.to_datetime(del_date).date()]
+                df_list.append(df)
+                if not df.empty:
+                    index_id_list.append(index_id)
+
+            end_date_dt = pd.to_datetime(end_date).date()
             index_list = ['sp500', 'dax30', 'n225']
             cm_index_list = ['sp500rmb', 'dax30rmb', 'n225rmb']
             for i, c in zip(index_list, cm_index_list):
-                cm_index = yahoo_index_price.copy()
-                df = cm_index[cm_index['index_id'] == i]
-                df = cm_index_price.set_index('datetime').join(df.drop(['_update_time'], axis=1).set_index('datetime'))
-                df = df.fillna(method='ffill')
-                df['close'] = df['close'] * df['usd_central_parity_rate']
+                df = yahoo_index_price[yahoo_index_price['index_id'] == i]
+                df = df.set_index('datetime').reindex(cm_index_price.index).fillna(method='ffill')
+                # 存进去人民币计价的数据
+                df['close'] *= cm_index_price['usd_central_parity_rate']
                 df = df[['close']]
-                df['datetime'] = df.index
-                df['open'] = float('Nan')
-                df['high'] = float('Nan')
-                df['low'] = float('Nan')
-                df['volume'] = float('Nan')
-                df['total_turnover'] = float('Nan')
                 df['index_id'] = c
-                df['ret'] = df.close / df.close.shift(1) -1
-                df_list.append(df.copy())
+                df['ret'] = df.close / df.close.shift(1) - 1
+                df = df.loc[[end_date_dt], :]
+                df_list.append(df.reset_index())
 
-            index_dic = {k: v for k, v in zip(
-                self._index_info_df['order_book_id'], self._index_info_df['index_id']) if k != 'not_available'}
-            rq_index_price['index_id'] = rq_index_price['order_book_id'].map(
-                lambda x: index_dic[x])
-            res = []
-            for index_i in list(set(rq_index_price['order_book_id'].tolist())):
-                # update gold index data from em not rq
-                if index_dic[index_i] == 'gold':
-                    continue 
-                dftmp = rq_index_price[rq_index_price['order_book_id'] == index_i].copy()
-                dftmp['ret'] = dftmp.close / dftmp.close.shift(1) - 1
-                res.append(dftmp)
-            rq_index_price = pd.concat(res)
-            df_list.append(yahoo_index_price)
-            df_list.append(cxindex_index_price)
-            df_list.append(rq_index_price)
+            df_list.append(cxindex_index_price[cxindex_index_price.datetime == end_date_dt])
+            # 存进去美元计价的数据
+            df_list.append(yahoo_index_price[yahoo_index_price.datetime == end_date_dt])
 
-            df = pd.concat(df_list).drop(['order_book_id'], axis=1)
-            df['datetime'] = pd.to_datetime(df['datetime'])
-            end_date = datetime.datetime.strptime(end_date, '%Y%m%d').date()
-            df = df[df.datetime == end_date]
-            self._data_helper._upload_basic(df, IndexPrice.__table__.name)
+            df = pd.concat(df_list, ignore_index=True)
+            # TODO: 改成原子提交下边两个操作
+            BasicDataApi().delete_index_price(index_id_list, del_date, end_date)
+            self._data_helper._upload_basic(df.replace([np.Inf, -np.Inf], np.nan), IndexPrice.__table__.name)
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
     def fund_rating_latest(self):
@@ -219,7 +276,7 @@ class BasicDataPart1(object):
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
     def rq_stock_price(self, start_date, end_date):
@@ -238,7 +295,7 @@ class BasicDataPart1(object):
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
     def rq_fund_ret(self, start_date, end_date):
@@ -268,7 +325,7 @@ class BasicDataPart1(object):
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
     def stock_info(self):
@@ -278,7 +335,43 @@ class BasicDataPart1(object):
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
+            traceback.print_exc()
+            return False
+
+    @staticmethod
+    def _lambda_func_index_component_1(em_id_map: Dict[str, str], x: str):
+        try:
+            return em_id_map[x]
+        except KeyError:
+            return x
+
+    def sector_info(self):
+        try:
+            df = pd.read_csv('./data/sector_info.csv', index_col=0)
+            df['sector_type'] = df['sector_type'].map({'industry': SectorType.industry.name, 'topic': SectorType.topic.name})
+            self._data_helper._upload_basic(df, SectorInfo.__table__.name, to_truncate=True)
+            return True
+        except Exception as e:
+            print(e)
+            traceback.print_exc()
+            return False
+
+    def sector_funds(self):
+        try:
+            index_info_df = BasicDataApi().get_index_info()
+            index_info_df = index_info_df[index_info_df.em_id.notna()]
+            index_info_df = index_info_df.set_index('em_id')
+            em_id_list: List[str] = index_info_df.index.to_list()
+            em_id_to_index_id: Dict[str, str] = index_info_df['index_id'].to_dict()
+
+            df = pd.read_csv('./data/sector_funds.csv', index_col=0)
+            df['csi_index_id'] = df['csi_index_id'].transform(lambda x: BasicDataPart1._lambda_func_index_component_1({em_id.split('.')[0]: em_id for em_id in em_id_list if em_id is not None}, x))
+            df['index_id'] = df['csi_index_id'].map(em_id_to_index_id)
+            self._data_helper._upload_basic(df, SectorFunds.__table__.name, to_truncate=True)
+            return True
+        except Exception as e:
+            print(e)
+            traceback.print_exc()
             return False
 
     def fund_size(self, start_date, end_date):
@@ -293,52 +386,24 @@ class BasicDataPart1(object):
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
-    def trading_day_list(self, start_date, end_date):
+    def trading_day_list(self, start_date, end_date, is_automatic_update=True):
         try:
+            # 如果是每日自动更新，我们需要去存T+1日的数据
+            if is_automatic_update:
+                start_date = end_date
+                end_date = ''
             df = RawDataApi().get_em_tradedates(start_date, end_date)
             df = df.rename(columns={'TRADEDATES': 'datetime'})
+            if is_automatic_update:
+                df = df.iloc[[1], :]
             self._data_helper._upload_basic(df, TradingDayList.__table__.name)
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
-            return False
-
-    def em_index_price(self, start_date, end_date):
-        try:
-            start_date = datetime.datetime.strptime(start_date, '%Y%m%d') - datetime.timedelta(days=5)
-            del_date = start_date + datetime.timedelta(days=1)
-            start_date = start_date.strftime('%Y%m%d')
-            index_info = BasicDataApi().get_index_info()
-            em_id_list = index_info['em_id'].dropna().tolist()
-            raw_index_price = RawDataApi().get_em_index_price(start_date, end_date, em_id_list)
-            res = [] 
-            index_id_list = []
-            for em_id in em_id_list:
-                df = raw_index_price[raw_index_price['em_id'] == em_id].copy()
-                index_id = index_info[index_info['em_id'] == em_id].index_id.values[0]
-                df.drop(['em_id'], axis = 1, inplace=True)
-                df['index_id'] = index_id
-                df['high'] = float('nan')
-                df['low'] = float('nan')
-                df['open'] = float('nan')
-                df['total_turnover'] = float('nan')
-                df['volume'] = float('nan')
-                df['ret'] = df.close / df.close.shift(1) - 1
-                res.append(df)
-                if not df.empty:
-                    index_id_list.append(index_id)
-            BasicDataApi().delete_index_price(index_id_list, del_date, end_date)
-            df = pd.concat(res, axis=0, sort=False)
-            df = df[df.datetime >= del_date.date()]
-            self._data_helper._upload_basic(df, IndexPrice.__table__.name)
-            return True
-        except Exception as e:
-            print(e)
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
     def fund_size_hold_rate_history(self):
@@ -400,9 +465,6 @@ class BasicDataPart1(object):
         if not self.fund_nav(start_date, end_date):
             failed_tasks.append('fund_nav')
 
-        if not self.em_index_price(start_date, end_date):
-            failed_tasks.append('em_index_price')
-
         if not self.trading_day_list(start_date, end_date):
             failed_tasks.append('trading_day_list')
 
@@ -410,12 +472,17 @@ class BasicDataPart1(object):
 
 
 if __name__ == '__main__':
-    bdp = BasicDataPart1()
+    bdp = BasicDataPart1(BasicDataHelper())
     # start_date = '20200428'
     # end_date = '20200428'
+    # bdp.fund_nav(start_date, end_date)
     # bdp.index_price(start_date, end_date)
-    # bdp.em_index_price(start_date, end_date)
     # bdp.rq_stock_price(start_date, end_date)
     # bdp.rq_fund_ret(start_date, end_date)
     # bdp.fund_rating_latest()
-    bdp.fund_size('20200513', '20200513')
+    # bdp.fund_size('20200513', '20200513')
+    # bdp.trading_day_list('20200124', '20200301', False)
+    # bdp.fund_benchmark()
+    # bdp.sector_info()
+    # bdp.sector_funds()
+    # bdp.index_component()
