@@ -22,16 +22,16 @@ class RqRawDataDownloader(object):
         self._fund_order_book_id_list = list(set(rq.fund.all_instruments()['order_book_id'].tolist()))
 
         # Get stock order_book_ids
-        self._stock_order_book_id_list = rq.all_instruments(type='CS', market='cn')['order_book_id'].tolist()
+        # self._stock_order_book_id_list = rq.all_instruments(type='CS', market='cn')['order_book_id'].tolist()
 
         # Get index list
-        index_df = BasicDataApi().get_index_info()
-        self._index_list = [order_book_id for order_book_id in index_df['order_book_id'].tolist() 
-            if order_book_id and order_book_id != 'not_available' and '.' in order_book_id]
-        self._future_list = [order_book_id for order_book_id in index_df['order_book_id'].tolist() 
-            if order_book_id and order_book_id != 'not_available' and '.' not in order_book_id]
-        self._order_book_id_2_index_id = {order_book_id : index_id for order_book_id, index_id in 
-            zip(index_df['order_book_id'], index_df['index_id']) if order_book_id != 'not_available'}
+        # index_df = BasicDataApi().get_index_info()
+        # self._index_list = [order_book_id for order_book_id in index_df['order_book_id'].tolist()
+        #     if order_book_id and order_book_id != 'not_available' and '.' in order_book_id]
+        # self._future_list = [order_book_id for order_book_id in index_df['order_book_id'].tolist()
+        #     if order_book_id and order_book_id != 'not_available' and '.' not in order_book_id]
+        # self._order_book_id_2_index_id = {order_book_id : index_id for order_book_id, index_id in
+        #     zip(index_df['order_book_id'], index_df['index_id']) if order_book_id != 'not_available'}
 
     def _get_date_list(self, start_date, end_date):
         start_datetime = datetime.datetime.strptime(start_date, '%Y%m%d')
@@ -44,7 +44,7 @@ class RqRawDataDownloader(object):
 
     def rq_fund_indicator(self, start_date, end_date):
         try:
-            df = rq.fund.get_indicators(self._fund_order_book_id_list, start_date, end_date, 
+            df = rq.fund.get_indicators(self._fund_order_book_id_list, start_date, end_date,
                 fields=FUND_INDICATORS)
 
             if df is not None:
@@ -53,7 +53,7 @@ class RqRawDataDownloader(object):
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
     def rq_fund_nav(self, start_date, end_date):
@@ -78,7 +78,7 @@ class RqRawDataDownloader(object):
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
     def rq_fund_size(self):
@@ -86,7 +86,7 @@ class RqRawDataDownloader(object):
             instruments = []
             for instrument in rq.fund.instruments(self._fund_order_book_id_list):
                 instruments.append({
-                    'order_book_id' : instrument.order_book_id, 
+                    'order_book_id' : instrument.order_book_id,
                     'latest_size' : instrument.latest_size
                 })
             df = pd.DataFrame(instruments)
@@ -95,7 +95,7 @@ class RqRawDataDownloader(object):
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
     def rq_index_price(self, start_date, end_date):
@@ -110,14 +110,14 @@ class RqRawDataDownloader(object):
             df = rq.get_price(self._future_list, start_date, end_date, expect_df=True)
             if df is not None:
                 df = df.reset_index().drop(['limit_down', 'prev_settlement', 'limit_up', 'open_interest', 'settlement', 
-                    'dominant_id'], axis = 1)
+                    'dominant_id'], axis = 1, errors='ignore')
                 df['datetime'] = df['date']
                 df = df.drop(['date'], axis = 1)
                 self._data_helper._upload_raw(df, RqIndexPrice.__table__.name)
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
     def rq_index_weight(self, start_date, end_date):
@@ -136,7 +136,7 @@ class RqRawDataDownloader(object):
                                 'index_id': self._order_book_id_2_index_id[i],
                                 'datetime': d,
                                 'weight_list': json.dumps(weight_list),
-                                'stock_list': json.dumps(stock_list),  
+                                'stock_list': json.dumps(stock_list),
                             })
                     except:
                         pass
@@ -146,7 +146,7 @@ class RqRawDataDownloader(object):
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
     def rq_stock_fin_fac(self, start_date, end_date):
@@ -163,12 +163,12 @@ class RqRawDataDownloader(object):
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
     def rq_stock_post_price(self, start_date, end_date):
         try:
-            df = rq.get_price(self._stock_order_book_id_list, start_date=start_date, end_date=end_date, 
+            df = rq.get_price(self._stock_order_book_id_list, start_date=start_date, end_date=end_date,
                 adjust_type='post', expect_df=True)
             if df is not None:
                 df = df.reset_index().rename(columns={'date': 'datetime'})
@@ -176,24 +176,24 @@ class RqRawDataDownloader(object):
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
     def rq_stock_price(self, start_date, end_date):
         try:
-            df = rq.get_price(self._stock_order_book_id_list, start_date=start_date, end_date=end_date, 
+            df = rq.get_price(self._stock_order_book_id_list, start_date=start_date, end_date=end_date,
                 adjust_type='none', expect_df=True)
             if df is not None:
                 df = df.reset_index().rename(columns={'date': 'datetime'})
                 self._data_helper._upload_raw(df, RqStockPrice.__table__.name)
             return True
         except:
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
     def rq_stock_valuation(self, start_date, end_date):
         try:
-            df = rq.get_factor(self._stock_order_book_id_list, STOCK_VALUATION_FACTORS, start_date=start_date, 
+            df = rq.get_factor(self._stock_order_book_id_list, STOCK_VALUATION_FACTORS, start_date=start_date,
                 end_date=end_date, expect_df=True)
             if df is not None:
                 df = df.reset_index().rename(columns={'order_book_id': 'stock_id', 'date':'datetime'}).replace(
@@ -202,19 +202,19 @@ class RqRawDataDownloader(object):
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
     def trading_day_list(self, start_date, end_date):
         try:
-            df = pd.DataFrame(rq.get_trading_dates(start_date=start_date, end_date=end_date, market='cn'), 
+            df = pd.DataFrame(rq.get_trading_dates(start_date=start_date, end_date=end_date, market='cn'),
                 columns=['datetime'])
             if df is not None:
                 self._data_helper._upload_raw(df, TradingDayList.__table__.name)
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
     def stock_info(self):
@@ -222,7 +222,7 @@ class RqRawDataDownloader(object):
             order_book_ids = []
             for stock_order_book_id in self._stock_order_book_id_list:
                 order_book_ids.append({
-                    'rq_id' : stock_order_book_id, 
+                    'rq_id' : stock_order_book_id,
                     'stock_id' : stock_order_book_id
                 })
             df = pd.DataFrame(order_book_ids)
@@ -231,7 +231,7 @@ class RqRawDataDownloader(object):
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
     def fund_rating(self, start_date, end_date):
@@ -244,7 +244,7 @@ class RqRawDataDownloader(object):
             return True
         except Exception as e:
             print(e)
-            traceback.print_stack()
+            traceback.print_exc()
             return False
 
     def download_all(self, start_date, end_date):
@@ -288,10 +288,10 @@ class RqRawDataDownloader(object):
         # # Safe to disable
         # if not self.rq_fund_size():
         #     failed_tasks.append('rq_fund_size')
-            
-        # TODO: gold
-        if not self.rq_index_price(start_date, end_date):
-            failed_tasks.append('rq_index_price')
+
+        # # Safe to disable
+        # if not self.rq_index_price(start_date, end_date):
+        #     failed_tasks.append('rq_index_price')
 
         # TODO: used in basic db, replace to EM
         if not self.fund_rating(start_date, end_date):
@@ -299,12 +299,12 @@ class RqRawDataDownloader(object):
 
         return failed_tasks
 
+
 if __name__ == "__main__":
     from ....util.config import SurfingConfigurator
 
     rq_license = SurfingConfigurator().get_license_settings('rq')
-    data_helper = RawDataHelper()
-    raw_data_downloader = RqRawDataDownloader(rq_license, data_helper)
+    raw_data_downloader = RqRawDataDownloader(rq_license, RawDataHelper())
     # raw_data_downloader.rq_index_price('20200421', '20200422')
     # raw_data_downloader.rq_index_weight('20200428', '20200428')
-    # raw_data_downloader.rq_stock_price('20050101', '20091231')
\ No newline at end of file
+    # raw_data_downloader.rq_stock_price('20050101', '20091231')
