@@ -2,7 +2,6 @@ import dataclasses
 import copy
 import datetime
 import numpy as np
-import copy
 
 @dataclasses.dataclass
 class AssetDict:
@@ -13,14 +12,9 @@ class AssetDict:
     sp500rmb: float=0.0
     national_debt: float=0.0
     gold: float=0.0
-    credit_debt: float=0.0
-    dax30rmb: float=0.0
-    real_state: float=0.0
-    oil: float=0.0
-    n225rmb: float=0.0
     cash: float=0.0
     mmf: float=0.0
-
+    hsi: float=0.0
     def __repr__(self):
         s = f'<{self.__class__.__name__}'
         for k, v in self.__dict__.items():
@@ -34,6 +28,18 @@ class AssetDict:
     def isnan(self, v):
         return v is None or np.isnan(v)
 
+@dataclasses.dataclass
+class AssetTimeSpan:
+    # 大类资产指标考察年数
+    hs300: int=3
+    csi500: int=3
+    gem: int=1
+    sp500rmb: int=1
+    national_debt: int=1
+    gold: int=1
+    mmf: int=1
+    hsi: int=1
+
 @dataclasses.dataclass
 class AssetWeight(AssetDict):
 
@@ -109,7 +115,7 @@ class AssetPosition(AssetDict):
         assert trd.is_buy or (self.__dict__[trd.index_id] - v > -1e-8), f'asset trade failed, (sell){v} (cur){self.__dict__[trd.index_id]} (trd){trd} (pos){self}'
         amt = trd.amount if trd.amount else v * p
         self.__dict__[trd.index_id] += v if trd.is_buy else -v
-        self.cash += (-amt if trd.is_buy else amt) - trd.commission
+        self.cash += (-amt if trd.is_buy else amt)
 
 @dataclasses.dataclass
 class AssetValue(AssetDict):
@@ -144,13 +150,15 @@ class AssetValue(AssetDict):
         wgts.rebalance()
         return wgts
         
-
 @dataclasses.dataclass
 class FundTrade(AssetTrade):
 
     fund_id: str=None
     fund_unit_nav: float=None
     fund_unit_volume: float=None
+    is_to_cleanup: bool=False
+    is_permitted_fund: bool=None
+    trigger: int=None # by TradeTrigger in constant.py
 
 @dataclasses.dataclass
 class FundPosItem:
@@ -160,13 +168,15 @@ class FundPosItem:
     volume: int             # 基金份额
     avg_cost: float=None    # 平均成本
     update_time: datetime.date=None # 更新时间
+    is_permitted_fund: bool=None  # 在白名单里或者没有白名单 True   不在白名单里或者在黑名单里  False    
 
     def update(self, trade: FundTrade):
         assert self.fund_id == trade.fund_id, f'fund trade failed, self.fund_id != trade.fund_id: self.fund_id {self.fund_id}  trade.fund_id {trade.fund_id}'
         p = trade.trade_price if trade.trade_price else trade.mark_price
         v = trade.volume if trade.volume else trade.amount / p
         a = trade.amount if trade.amount else v * p
-        assert trade.is_buy or (self.volume - v > -1e-8), f'fund trade failed, trade.volume {v} more than self.volume {self.volume}'
+        assert trade.is_buy or trade.is_to_cleanup or (self.volume - v > -1e-8), f'fund trade failed, trade.volume {v} more than self.volume {self.volume} on {self.fund_id}'
+
         vol_diff = v if trade.is_buy else -v
         amt_diff = a if trade.is_buy else -a
         volume = self.volume + vol_diff
@@ -175,7 +185,12 @@ class FundPosItem:
             self.volume = volume
             self.avg_cost = amt / volume
         else:
-            self.volume = volume
+            if trade.is_to_cleanup:
+                trade.volume = self.volume
+                trade.amount = trade.volume * p
+                self.volume = 0
+            else:
+                self.volume = volume
             if self.volume < 1e-8:# 清空仓位 平均成本为0
                 self.avg_cost = 0
             else: #非清仓卖出 平均成本不变
@@ -230,15 +245,21 @@ class FundTradeParam:
     PurchaseDiscount: float=0.15 # 基金的买入折扣
     RedeemDiscount: float=1.0    # 基金的卖出折扣
 
+    # new trading opt
+    JudgeIndexDiff: float=0.06
+    JudgeFundSelection: float=0.4 # 0-1，越大表示对基金选择的分数要求越高
+    JudgeFundRebalance: float=0.8 # 0-1，越大表示对基金比例分配分散度要求越高
+    DiffJudgeAssetWgtRequirement: float=0.01 # 比例超过该值的资产内的基金异常可以触发调仓
+
 @dataclasses.dataclass
 class TAAParam:
 
-    HighThreshold: float=0.9    # 超过多少算作进入估值偏高的区域
-    LowThreshold: float=0.1     # 低于多少算作进入估值偏低的区域
+    HighThreshold: float=0.95    # 超过多少算作进入估值偏高的区域
+    LowThreshold: float=-1     # 低于多少算作进入估值偏低的区域
     HighMinus:    float=0.05    # 估值偏高时应该调低的占比
-    LowPlus:      float=0.05    # 估值偏低是应该调高的占比
+    LowPlus:      float=0   # 估值偏低是应该调高的占比
     HighStop:     float=0.5     # 进入估值偏高区后，什么时候算作退出
-    LowStop:      float=0.5     # 进入估值偏低区后，什么时候算作退出
+    LowStop:      float=-1     # 进入估值偏低区后，什么时候算作退出
     TuneCash:     bool=False    # 默认taa调仓时，不调整cash
     ToMmf:        bool=True    # 默认taa加减仓不改变货基权重
 
@@ -256,7 +277,23 @@ class FundScoreParam:
 @dataclasses.dataclass
 class FAParam:
 
-    MaxFundNumUnderAsset: float=3 # 同一个大类资产下基金最多数
+    MaxFundNumUnderAsset: float=4 # 同一个大类资产下基金最多数
+
+@dataclasses.dataclass
+class AssetTradeCache:
+    
+    index_id: str
+    index_tar_wgt: float=None
+    index_cur_wgt: float=None
+    cur_fund_ids: set=None
+    fund_scores: list=None
+    fund_ranks: dict=None
+    proper_fund_num: float=None
+    fund_cur_wgts: dict=None
+    fund_judge_ranking_score: float=None # 当前“基金选择”分数
+    fund_judge_ranking_best: float=None  # “基金选择”最优分数
+    fund_judge_ranking: float=None       # 0-1 之间，衡量基金选择分数
+    fund_judge_diverse: float=None       # 0-1 之间，衡量基金配置比例
 
 @dataclasses.dataclass
 class FundWeightItem:
@@ -270,6 +307,10 @@ class FundWeightItem:
     def __post_init__(self):
         self.fund_wgt = self.fund_wgt or self.asset_wgt * self.fund_wgt_in_asset
 
+    def add_tot_wgt(self, tot_wgt_to_add):
+        self.fund_wgt += tot_wgt_to_add
+        self.fund_wgt_in_asset = self.fund_wgt / self.asset_wgt
+
     def __repr__(self):
         return f'<{self.fund_id} (wgt){self.fund_wgt} (index){self.index_id} (iwgt){self.asset_wgt} (fwgt){self.fund_wgt_in_asset}>'
 
@@ -277,14 +318,22 @@ class FundWeight:
 
     def __init__(self):
         self.funds = {} # fund_id -> FundWeightItem
+        self.index_fund_list = {} # index_id -> [fund_id]
 
     def add(self, fund_wgt_item: FundWeightItem):
+        index_id = fund_wgt_item.index_id
         self.funds[fund_wgt_item.fund_id] = fund_wgt_item
+        if index_id not in self.index_fund_list:
+            self.index_fund_list[index_id] = []
+        self.index_fund_list[index_id].append(fund_wgt_item.fund_id)
     
     def get_wgt(self, fund_id: str):
         fund_wgt_item = self.funds.get(fund_id)
         return fund_wgt_item.fund_wgt if fund_wgt_item else 0
     
+    def get_funds(self, index_id):
+        return self.index_fund_list.get(index_id, [])
+
     def get(self, fund_id: str):
         return self.funds.get(fund_id)
 
@@ -297,16 +346,15 @@ class FundWeight:
 
 class FundPosition:
 
-    def __init__(self, cash):
-        self.funds = {} # fund_id -> FundPosItem
-        self.index_fund_list = {} # index_id -> [fund_id]
+    def __init__(self, cash, funds=None, index_fund_list=None):
+        self.funds = funds or {} # fund_id -> FundPosItem
+        self.index_fund_list = index_fund_list or {} # index_id -> [fund_id]
         self.cash = cash
 
     def init_by_item_dict(self, cash, funds, index_fund_list):
         self.cash = cash
         self.funds = funds
         self.index_fund_list = index_fund_list
-        return self
 
     def convert_to_item_dict(self):
         return {
@@ -318,11 +366,10 @@ class FundPosition:
     def calc_mv_n_w(self, 
                     fund_navs: dict, # fund_id -> nav(adjusted net value)
                     index_id=None): # filter: 只计算某个index_id
-                
         mv = 0
         w_dic = {}
         for pos_item in self.funds.values():
-            if index_id and pos_item.index_id != index_id:
+            if index_id and pos_item.index_id != index_id or pos_item.volume == 0:
                 continue
             nav = fund_navs.get(pos_item.fund_id, np.nan)
             if np.isnan(nav):
@@ -330,7 +377,9 @@ class FundPosition:
             else:
                 mv += nav * pos_item.volume
             w_dic[pos_item.fund_id] = nav * pos_item.volume
-        mv = self.cash + mv
+            
+        if index_id is None:
+            mv = self.cash + mv
         w_dic = { f: w / mv for f, w in w_dic.items() }
         return mv, w_dic
 
@@ -356,8 +405,8 @@ class FundPosition:
     def get_funds(self, index_id):
         return self.index_fund_list.get(index_id, [])
 
-    def add(self, fund_id, index_id, volume, avg_cost, update_time):
-        pos_item = FundPosItem(fund_id=fund_id, index_id=index_id, volume=volume, avg_cost=avg_cost, update_time=update_time)
+    def add(self, fund_id, index_id, volume, avg_cost, update_time, is_permitted_fund):
+        pos_item = FundPosItem(fund_id=fund_id, index_id=index_id, volume=volume, avg_cost=avg_cost, update_time=update_time, is_permitted_fund=is_permitted_fund)
         self.funds[fund_id] = pos_item
         if index_id not in self.index_fund_list:
             self.index_fund_list[index_id] = []
@@ -369,26 +418,29 @@ class FundPosition:
         pos_item = self.funds.get(trade.fund_id)
         if pos_item is None:
             assert trade.is_buy, "fund volume cannot be negative"
-            pos_item = self.add(trade.fund_id, trade.index_id, 0, 0, trade.trade_date or trade.submit_date)
-            
+            pos_item = self.add(trade.fund_id, trade.index_id, 0, 0, trade.trade_date or trade.submit_date, trade.is_permitted_fund)
+
+        p = trade.trade_date if trade.trade_price else trade.mark_price
+        amt = trade.amount if trade.amount else trade.volume * p
+        cash_left = self.cash + (-amt if trade.is_buy else amt) 
+        if cash_left <= -1e-8:
+            print(f'cash negative alert: cancel order due to cash negative trade: {trade}, cash left if trade successed: {cash_left}')
+            return False
         if pos_item.update(trade):
-            p = trade.trade_date if trade.trade_price else trade.mark_price
-            amt = trade.amount if trade.amount else trade.volume * p
-            cash_left = self.cash + (-amt if trade.is_buy else amt) - trade.commission 
-            if cash_left <= -1e-8:
-                print(f'cash negative alert: cancel order due to cash negative trade: {trade}, cash left if trade successed: {cash_left}')
-                return False
             self.cash = cash_left
             return True
         else:
             return False
 
     def copy(self):
-        n = FundPosition(cash=self.cash)
-        n.funds = copy.deepcopy(self.funds)
-        n.index_fund_list = copy.deepcopy(self.index_fund_list)
-        return n
+        return FundPosition(
+            cash=self.cash, 
+            funds=copy.deepcopy(self.funds), 
+            index_fund_list=copy.deepcopy(self.index_fund_list)
+        )
 
+    '''
+    # depreciated
     def calc_weight(self, 
                 fund_navs: dict, # fund_id -> nav(adjusted net value)
                 index_id=None): # filter: 只计算某个index_id
@@ -402,6 +454,7 @@ class FundPosition:
             else:
                 mv += nav * pos_item.volume
         return mv
+    '''
 
     def __repr__(self):
         s = f'<FP cash={self.cash}'
@@ -479,6 +532,7 @@ class ScorePenaltyParams:
     FilterYearIndex:list = None
     Penalty:float = -0.5 
     JudgeYearLength:float = 1.0
+    BlackListPenalty:float = - 10000
 
     def __post_init__(self):
         self.FilterYearIndex = self.FilterYearIndex or ['national_debt', 'csi500']
