@@ -1,7 +1,10 @@
-from sqlalchemy import CHAR, Column, Integer, Index, BOOLEAN, text, TEXT
+
+from sqlalchemy import CHAR, Column, Integer, Index, BOOLEAN, text, TEXT, Enum
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.dialects.mysql import DOUBLE, DATE, TINYINT, DATETIME
 
+from ...constant import SectorType, IndexPriceSource
+
 
 class Base():
     _update_time = Column('_update_time', DATETIME, nullable=False, server_default=text('CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP'))  # 更新时间
@@ -12,19 +15,62 @@ Base._update_time._creation_order = 9999
 Base = declarative_base(cls=Base)
 
 
+class SectorInfo(Base):
+    '''板块信息表'''
+
+    __tablename__ = 'sector_info'
+
+    sector_id = Column(CHAR(64), primary_key=True)  # 板块ID
+    sector_type = Column(Enum(SectorType), nullable=False)  # 板块类型 industry-行业 topic-主题
+    sector_desc = Column(TEXT, nullable=False)  # 板块描述
+    sector_name = Column(CHAR(16), nullable=False)  # 板块名称
+    main_index_id = Column(CHAR(16), nullable=False)  # 板块代表指数
+
+
+class SectorFunds(Base):
+    '''板块基金表'''
+
+    __tablename__ = 'sector_funds'
+
+    id = Column(Integer, primary_key=True)
+
+    sector_id = Column(CHAR(64), nullable=False)  # 板块ID
+    csi_index_id = Column(CHAR(16), nullable=False)  # Choice指数ID
+    index_id = Column(CHAR(20), nullable=False)  # 指数ID
+    em_fund_id = Column(CHAR(16))  # Choice基金ID
+    fund_id = Column(CHAR(16))  # 基金ID
+
+
 class IndexInfo(Base):
     '''指数信息表'''
 
     __tablename__ = 'index_info'
     index_id = Column(CHAR(20), primary_key=True)
 
-    order_book_id = Column(CHAR(20)) # 米筐ID
+    order_book_id = Column(CHAR(20))  # 米筐ID
     # web_id= Column(CHAR(20)) # 数据所在网页ID
-    industry_tag = Column(CHAR(64)) # 行业标签
-    tag_method = Column(CHAR(64)) # 估值评分采用方法
-    desc_name = Column(CHAR(64)) # 名称
-    em_id = Column(CHAR(20)) # ChoiceID
-    em_plate_id = Column(CHAR(20)) # Choice板块ID
+    industry_tag = Column(CHAR(64))  # 行业标签
+    tag_method = Column(CHAR(64))  # 估值评分采用方法
+    desc_name = Column(CHAR(64), nullable=False)  # 名称
+    maker_name = Column(CHAR(32), nullable=False)  # 编制机构
+    em_id = Column(CHAR(20))  # ChoiceID
+    em_plate_id = Column(CHAR(20))  # Choice板块ID
+    index_profile = Column(TEXT, nullable=False)  # 指数概况
+    price_source = Column(Enum(IndexPriceSource), nullable=False)  # 指数价格来源标记
+
+
+class IndexComponent(Base):
+
+    __tablename__ = 'index_component'
+
+    index_id = Column(CHAR(20), primary_key=True)
+    datetime = Column(DATE, primary_key=True)  # 日期
+    num = Column(Integer, nullable=False)  # 成分股数量
+    id_cat = Column(Enum(SectorType), nullable=False)  # 所属类型 industry-行业 topic-主题
+    sector = Column(TEXT, nullable=False)  # 所属板块
+    top10 = Column(TEXT, nullable=False)  # 前10成分及权重
+    related_funds = Column(TEXT)  # 相关产品
+    all_constitu = Column(TEXT, nullable=False)  # 所有成分及权重(from Choice)
 
 
 class StockInfo(Base):
@@ -86,6 +132,25 @@ class FundInfo(Base):
     ac_filter = Column("ac_filter", BOOLEAN, nullable=False) #是否ac均有要排除的a 0 排除  1 保留  
     is_selected_mmf = Column(TINYINT(1)) # 是否基金规模最大的前50只货币基金(10年前有两只存在)  0 非  1 是
     national_debt_extension = Column(TINYINT(1)) # 纯债tag  0 非  1 是
+    index_id_new = Column(CHAR(20)) # 基于第一业绩标准所标注的指数ID新
+
+class FundBenchmark(Base):
+    '''基金业绩比较基准表'''
+
+    __tablename__ = 'fund_benchmark'
+
+    em_id = Column(CHAR(16), primary_key=True)  # 基金代码
+    fund_id = Column(CHAR(16))  # 内部基金代码
+    # datetime = Column(DATE, primary_key=True)  # 日期
+    index_text = Column(TEXT, nullable=False)  # 业绩比较基准
+    benchmark = Column(TEXT, nullable=False)  # 解析出来的业绩比较基准
+    assets = Column(CHAR(32))  # 追踪的标的
+    industry = Column(CHAR(64))  # 行业分类
+
+    # __table_args__ = (
+    #     Index('idx_em_fund_benchmark_datetime', 'datetime'),
+    # )
+
 
 class TradingDayList(Base):
     '''交易日列表'''
@@ -135,7 +200,6 @@ class IndexPrice(Base):
         Index('idx_index_price_datetime', 'datetime'),
     )
 
-
 class StockPrice(Base):
     '''股票价格表'''
 
@@ -225,5 +289,3 @@ class Fund_size_and_hold_rate(Base):
     size = Column(DOUBLE(asdecimal=False)) # 最新规模
     personal_holds = Column(DOUBLE(asdecimal=False)) # 个人持有比例 单位百分比 
     institution_holds = Column(DOUBLE(asdecimal=False)) # 机构持有比例 单位百分比 
-
-    
\ No newline at end of file
