@@ -1,7 +1,9 @@
 
-from sqlalchemy import CHAR, Column, Integer, TEXT, BOOLEAN, text, Index
+from sqlalchemy import CHAR, Column, Integer, TEXT, BOOLEAN, text, Index, Enum
 from sqlalchemy.ext.declarative import declarative_base
-from sqlalchemy.dialects.mysql import DOUBLE, DATE, TINYINT, DATETIME
+from sqlalchemy.dialects.mysql import DOUBLE, DATE, TINYINT, DATETIME, MEDIUMTEXT, YEAR
+
+from ...constant import SectorType, IndexPriceSource, CodeFeeMode
 
 
 class Base():
@@ -224,6 +226,18 @@ class EmIndexPrice(Base):
     datetime = Column(DATE, primary_key=True) # 日期
     close = Column(DOUBLE(asdecimal=False)) # 收盘价
 
+class EmIndexInfo(Base):
+    '''Choice指数信息'''
+
+    __tablename__ = 'em_index_info'
+
+    CODES = Column('em_id', CHAR(20), primary_key=True)  # Choice代码
+
+    NAME = Column('name', CHAR(64), nullable=False)  # 指数名称
+    SHORTNAME = Column('short_name', CHAR(32), nullable=False)  # 指数简称
+    MAKERNAME = Column('maker_name', CHAR(32), nullable=False)  # 编制机构
+    INDEXPROFILE = Column('index_profile', TEXT, nullable=False)  # 指数概况
+    price_source = Column(Enum(IndexPriceSource), nullable=False)  # 指数价格来源标记
 
 class FundFee(Base):
     '''基金费率'''
@@ -629,6 +643,22 @@ class EmStockRefinancing(Base):
     ATTACHNAME = Column('attach_name', TEXT) # 原始公告链接
     ADDPURPOSE = Column('add_purpose', TEXT, nullable=False) # 增发目的
 
+class EmStockEstimateFac(Base):
+    '''Choice预测因子数据'''
+
+    __tablename__ = 'em_stock_estimate_fac'
+
+    CODES = Column('stock_id', CHAR(16), primary_key=True) # EM股票ID
+    DATES = Column('datetime', DATE, primary_key=True) # 日期
+    predict_year = Column(YEAR, nullable=False) # 预测年度
+    ESTGR = Column('est_gr', DOUBLE(asdecimal=False), nullable=False) # 预测营业总收入平均值
+    ESTNI = Column('est_ni', DOUBLE(asdecimal=False), nullable=False) # 预测归属于母公司的净利润平均值
+
+    __table_args__ = (
+        Index('idx_em_stock_estimate_fac_datetime', 'datetime'),
+        Index('idx_em_stock_estimate_fac_predict_year', 'predict_year'),
+    )
+
 class EmIndexVal(Base):
     '''Choice指数估值数据'''
 
@@ -643,6 +673,7 @@ class EmIndexVal(Base):
     PCFTTM = Column("pcf_ttm", DOUBLE(asdecimal=False)) # 市现率PCF(TTM)
     ROE = Column("roe", DOUBLE(asdecimal=False)) # 净资产收益率
     EPSTTM = Column("eps_ttm", DOUBLE(asdecimal=False)) # 每股收益TTM
+    ESTPEG = Column("est_peg",DOUBLE(asdecimal=False)) # 预测PEG
 
 class EmTradeDates(Base):
     '''Choice交易日数据'''
@@ -661,12 +692,228 @@ class EmIndexComponent(Base):
     em_plate_id = Column(CHAR(20), nullable=False)  # Choice板块ID
     stock_list = Column(TEXT, nullable=False)  # 股票列表
 
+class CSIndexComponent(Base):
+
+    __tablename__ = 'cs_index_component'
+
+    index_id = Column(CHAR(20), primary_key=True)
+    datetime = Column(DATE, primary_key=True)  # 日期
+    num = Column(Integer, nullable=False)  # 成分股数量
+    id_cat = Column(Enum(SectorType), nullable=False)  # 所属类型 industry-行业 topic-主题
+    sector = Column(TEXT, nullable=False)  # 所属板块
+    top10 = Column(TEXT, nullable=False)  # 前10成分及权重
+    related_funds = Column(TEXT)  # 相关产品
+    all_constitu = Column(TEXT, nullable=False)  # 所有成分及权重(from Choice)
+
 class EmFundHoldingRate(Base):
     '''Choice基金持有人比例'''
     __tablename__ = 'em_fund_holding_rate'
 
-    CODES =Column(CHAR(10), primary_key=True) # 合约代码
+    CODES = Column(CHAR(10), primary_key=True) # 合约代码
     DATES = Column(DATE, primary_key=True) # 日期
     HOLDPERSONALHOLDINGPCT = Column(DOUBLE(asdecimal=False)) # 个人持有比例 单位百分比
     HOLDINSTIHOLDINGPCT = Column(DOUBLE(asdecimal=False)) # 机构持有比例 单位百分比
 
+class EmFundList(Base):
+    '''Choice基金列表'''
+
+    __tablename__ = 'em_fund_list'
+
+    datetime = Column(DATE, primary_key=True) # 日期
+    all_live_fund_list = Column(MEDIUMTEXT, nullable=False) # 全部基金列表(不包含未成立、已到期)
+    delisted_fund_list = Column(MEDIUMTEXT, nullable=False) # 已摘牌基金列表
+
+class EmFundInfo(Base):
+    '''Choice基金信息表'''
+
+    __tablename__ = 'em_fund_info'
+
+    CODES = Column('em_id', CHAR(16), primary_key=True) # 基金代码
+    NAME = Column('name', CHAR(32), nullable=False) # 基金名称
+    FNAME = Column('full_name', CHAR(128), nullable=False) # 基金全称
+    FOUNDDATE = Column('start_date', DATE) # 成立日期
+    MATURITYDATE = Column('end_date', DATE) # 结束日期
+    FIRSTINVESTTYPE = Column('invest_type_class_1', CHAR(32), nullable=False) # 投资类型（一级）
+    SECONDINVESTTYPE = Column('invest_type_class_2', CHAR(32), nullable=False) # 投资类型（二级）
+    TRADECUR = Column('currency', CHAR(16), nullable=False) # 交易币种
+    STRUCFUNDORNOT = Column('is_structured', BOOLEAN, nullable=False) # 是否分级基金
+    GFCODEM = Column('base_fund_id', CHAR(16)) # 分级基金母基金代码
+    FUNDTYPE = Column('is_open', BOOLEAN, nullable=False) # 是否定期开放基金, 以Choice的FUNDTYPE来判断
+    PREDFUNDMANAGER = Column('manager_id', TEXT, nullable=False) # 基金经理(历任)
+    MGRCOMP = Column('company_id', CHAR(64), nullable=False) # 基金公司名称
+
+class EmFundBenchmark(Base):
+    '''Choice基金业绩比较基准表'''
+
+    __tablename__ = 'em_fund_benchmark'
+
+    CODES = Column('em_id', CHAR(16), primary_key=True) # 基金代码
+    DATES = Column('datetime', DATE, primary_key=True) # 日期
+    BENCHMARK = Column('benchmark', CHAR(255), nullable=False) # 业绩比较基准
+
+    __table_args__ = (
+        Index('idx_em_fund_benchmark_datetime', 'datetime'),
+    )
+
+class EmFundFee(Base):
+    '''基金费率'''
+
+    __tablename__ = 'em_fund_fee'
+
+    CODES = Column('em_id', CHAR(16), primary_key=True)  # 基金代码
+    MANAGFEERATIO = Column('manage_fee', TEXT)  # 管理费率
+    CUSTODIANFEERATIO = Column('trustee_fee', TEXT)  # 托管费率
+    SUBSCRFEERATIO = Column('subscr_fee', TEXT)  # 认购费率
+    PURCHFEERATIO = Column('purchase_fee', TEXT)  # 申购费率
+    REDEMFEERATIO = Column('redeem_fee', TEXT)  # 赎回费率
+    SERVICEFEERATIO = Column('service_fee', TEXT)  # 销售服务费率
+    CODEFEEMODE = Column('code_fee_mode', Enum(CodeFeeMode), nullable=False)  # 代码适用收费模式
+
+class WindFundHolderStructure(Base):
+    '''万德静态数据 中国共同基金持有人结构'''
+
+    __tablename__ = 'wind_fund_holder_structure'
+    
+    END_DT = Column(CHAR(8), primary_key=True)  # 日期
+    S_INFO_WINDCODE = Column(CHAR(20), primary_key=True)  # wind id
+    SEC_ID = Column(CHAR(15))  # 证券id
+    ANN_DT = Column(CHAR(8))  # 报告日
+    SCOPE = Column(CHAR(8))  # 范围
+    HOLDER_NUMBER = Column(DOUBLE(asdecimal=False)) # 基金份额持有人户数（户）
+    HOLDER_AVGHOLDING = Column(DOUBLE(asdecimal=False)) # 平均每户持有基金份额（份）
+    HOLDER_INSTITUTION_HOLDING = Column(DOUBLE(asdecimal=False)) # 机构投资者持有份额(份)
+    HOLDER_INSTITUTION_HOLDINGPCT = Column(DOUBLE(asdecimal=False)) # 机构投资者持有份额占比(%)
+    HOLDER_PERSONAL_HOLDING = Column(DOUBLE(asdecimal=False)) # 个人投资者持有份额(份)
+    HOLDER_PERSONAL_HOLDINGPCT = Column(DOUBLE(asdecimal=False)) # 个人投资者持有份额占比(%)
+    HOLDER_MNGEMP_HOLDING = Column(DOUBLE(asdecimal=False)) # 管理人员工持有份额(份)
+    HOLDER_MNGEMP_HOLDINGPCT = Column(DOUBLE(asdecimal=False)) # 管理人员工持有份额占比(%)
+    HOLDER_FEEDER_HOLDING = Column(DOUBLE(asdecimal=False)) # 联接基金持有份额(份)
+    HOLDER_FEEDER_HOLDINGPCT = Column(DOUBLE(asdecimal=False)) # 联接基金持有份额占比(%)
+    PUR_COST = Column(DOUBLE(asdecimal=False)) # 报告期买入股票成本总额(元)
+    SELL_INCOME = Column(DOUBLE(asdecimal=False)) # 报告期卖出股票收入总额(元)
+
+class WindFundStockPortfolio(Base):
+    '''万德静态数据 中国共同基金投资组合 持股明细'''
+
+    __tablename__ = 'wind_fund_stock_portfolio'
+
+    F_PRT_ENDDATE = Column(CHAR(8), primary_key=True)  # 日期
+    S_INFO_STOCKWINDCODE = Column(CHAR(10), primary_key=True)  # 万德股票id
+    S_INFO_WINDCODE = Column(CHAR(20), primary_key=True)  # wind id
+    CRNCY_CODE = Column(CHAR(5))  # 货币
+    F_PRT_STKVALUE = Column(DOUBLE(asdecimal=False)) # 持有股票市值 元
+    F_PRT_STKQUANTITY = Column(DOUBLE(asdecimal=False)) # 持有股票数量 股
+    F_PRT_STKVALUETONAV = Column(DOUBLE(asdecimal=False)) # 持有股票市值占基金净值比例
+    F_PRT_POSSTKVALUE = Column(DOUBLE(asdecimal=False)) # 积极投资持有股票市值 元
+    F_PRT_POSSTKQUANTITY = Column(DOUBLE(asdecimal=False)) # 积极投资持有股数 股
+    F_PRT_POSSTKTONAV = Column(DOUBLE(asdecimal=False)) # 积极投资持有股票市值占净资产比例
+    F_PRT_PASSTKEVALUE = Column(DOUBLE(asdecimal=False)) # 指数投资持有股票市值 元
+    F_PRT_PASSTKQUANTITY = Column(DOUBLE(asdecimal=False)) # 指数投资持有股数 股
+    F_PRT_PASSTKTONAV = Column(DOUBLE(asdecimal=False)) # 指数投资持有股票市值占净资产比例 
+    ANN_DATE = Column(CHAR(8))  # 公告日期
+    STOCK_PER = Column(DOUBLE(asdecimal=False)) # 占股票市值比
+    FLOAT_SHR_PER = Column(DOUBLE(asdecimal=False)) # 占流通股本比例
+
+class WindFundNav(Base):
+    '''万德静态数据 中国共同基金净值'''
+
+    __tablename__ = 'wind_fund_nav'
+
+    F_INFO_WINDCODE =  Column(CHAR(20), primary_key=True)  # wind id
+    PRICE_DATE = Column(CHAR(8), primary_key=True)  # 日期
+    ANN_DATE = Column(CHAR(8))  # 公告日期
+    F_NAV_UNIT = Column(DOUBLE(asdecimal=False)) # 单位净值
+    F_NAV_ACCUMULATED = Column(DOUBLE(asdecimal=False)) # 累积净值
+    F_NAV_DIVACCUMULATED = Column(DOUBLE(asdecimal=False)) # 累积分红
+    F_NAV_ADJFACTOR = Column(DOUBLE(asdecimal=False)) # 复权因子
+    CRNCY_CODE = Column(CHAR(5)) # 货币
+    F_PRT_NETASSET = Column(DOUBLE(asdecimal=False)) # 资产净值
+    F_ASSET_MERGEDSHARESORNOT = Column(DOUBLE(asdecimal=False)) # 是否合计数据
+    NETASSET_TOTAL = Column(DOUBLE(asdecimal=False)) # 合计资产净值
+    F_NAV_ADJUSTED = Column(DOUBLE(asdecimal=False)) # 复权单位净值
+    IS_EXDIVIDENDDATE = Column(DOUBLE(asdecimal=False)) # 是否净值除权日
+    F_NAV_DISTRIBUTION = Column(DOUBLE(asdecimal=False)) # 累积单位分配
+
+class WindFundManager(Base):
+    '''万德静态数据 中国共同基金基金经理表'''
+
+    __tablename__ = 'wind_fund_manager'
+
+    F_INFO_FUNDMANAGER_ID = Column(CHAR(10), primary_key=True) # 基金经理ID
+    ANN_DATE = Column(CHAR(8), primary_key=True) # 报告日期
+    F_INFO_WINDCODE = Column(CHAR(20)) # 万德基金ID
+    F_INFO_FUNDMANAGER = Column(CHAR(10)) # 姓名
+    F_INFO_MANAGER_GENDER = Column(CHAR(1)) # 性别
+    F_INFO_MANAGER_BIRTHYEAR = Column(CHAR(4)) # 出生年份
+    F_INFO_MANAGER_EDUCATION = Column(CHAR(10)) # 学历
+    F_INFO_MANAGER_NATIONALITY = Column(CHAR(10)) # 国籍
+    F_INFO_MANAGER_STARTDATE = Column(CHAR(8)) # 任职日期
+    F_INFO_MANAGER_LEAVEDATE = Column(CHAR(8)) # 离职日期 
+    F_INFO_MANAGER_RESUME = Column(TEXT) # 简历
+    S_INFO_MANAGER_POST = Column(CHAR(255)) # 备注（对于特殊基金经理的备注）
+    F_INFO_ESCROW_FUNDMANAGER = Column(CHAR(20)) # 代管基金经理
+    F_INFO_ESCROW_STARTDATE = Column(CHAR(8)) # 代管起始日期
+    F_INFO_ESCROW_LEAVEDATE = Column(CHAR(8)) # 代管结束日期
+    F_INFO_DIS_SERIAL_NUMBER = Column(CHAR(5)) # wind 展示序号
+
+class WindIndPortfolio(Base):
+    '''万德静态数据 中国共同基金基金持仓行业明细'''
+
+    __tablename__ = 'wind_ind_portfolio'
+
+    S_INFO_WINDCODE = Column(CHAR(20), primary_key=True) # 万德基金ID
+    F_PRT_ENDDATE = Column(CHAR(8), primary_key=True) # 日期
+    S_INFO_CSRCINDUSCODE = Column(CHAR(20), primary_key=True) # 证监会行业编号
+    F_PRT_INDUSVALUE = Column(DOUBLE(asdecimal=False)) # 持有行业市值 元
+    F_PRT_INDUSTONAV = Column(DOUBLE(asdecimal=False)) # 持有行业市值占基金净值比例 %
+    F_PRT_INDPOSVALUE = Column(DOUBLE(asdecimal=False)) # 积极投资持有行业市值 元
+    F_PRT_INDPOSPRO = Column(DOUBLE(asdecimal=False)) # 积极投资持有行业比例 %
+    F_PRT_INDPASSIVEVALUE = Column(DOUBLE(asdecimal=False)) # 指数投资持有行业市值 元
+    F_PRT_INDPASSIVEPRO = Column(DOUBLE(asdecimal=False)) # 指数投资持有行业比例 %
+    F_PRT_INDUSTONAVCHANGE = Column(DOUBLE(asdecimal=False)) # 持有行业市值比例较上期变化 %
+    S_INFO_CSRCINDUSNAME = Column(CHAR(50)) # 行业名称
+    F_ANN_DATE = Column(CHAR(8)) # 公告日期
+
+class FundBenchmark(Base):
+    '''基金业绩比较基准表'''
+
+    __tablename__ = 'fund_benchmark'
+
+    em_id = Column(CHAR(16), primary_key=True)  # 基金代码
+    fund_id = Column(CHAR(16))  # 内部基金代码
+    # datetime = Column(DATE, primary_key=True)  # 日期
+    index_text = Column(TEXT, nullable=False)  # 业绩比较基准
+    benchmark = Column(TEXT, nullable=False)  # 解析出来的业绩比较基准
+    assets = Column(CHAR(32))  # 追踪的标的
+    industry = Column(CHAR(64))  # 行业分类 
+
+
+class EMFundRate(Base):
+    '''东财基金评级'''
+
+    __tablename__ = 'em_fund_rate'
+    
+    CODES = Column(CHAR(20), primary_key=True) # 东财基金ID
+    DATES = Column(DATE, primary_key=True) # 日期
+    MKTCOMPRE3YRAT = Column(CHAR(15)) # 市场综合3年评级
+    MORNSTAR3YRAT = Column(CHAR(15)) # 晨星3年评级
+    MORNSTAR5YRAT = Column(CHAR(15)) # 晨星5年评级
+    MERCHANTS3YRAT = Column(CHAR(15)) # 招商3年评级
+    TXCOMRATRAT = Column(CHAR(15)) # 天相投顾综合评级
+    SHSTOCKSTAR3YCOMRAT = Column(CHAR(15)) #上海证券3年评级(综合评级)
+    SHSTOCKSTAR3YSTOCKRAT = Column(CHAR(15)) #上海证券3年评级(选证能力)
+    SHSTOCKSTAR3YTIMERAT = Column(CHAR(15)) # 上海证券3年评级(择时能力) 
+    SHSTOCKSTAR3YSHARPERAT = Column(CHAR(15)) # 上海证券3年评级(夏普比率)
+    SHSTOCKSTAR5YCOMRATRAT = Column(CHAR(15)) # 上海证券5年评级(综合评级) 
+    SHSTOCKSTAR5YSTOCKRAT = Column(CHAR(15)) # 上海证券5年评级(选证能力)
+    SHSTOCKSTAR5YTIMERAT = Column(CHAR(15)) # 上海证券5年评级(择时能力) 
+    SHSTOCKSTAR5YSHARPRAT = Column(CHAR(15)) # 上海证券5年评级(夏普比率)
+    JAJXCOMRAT = Column(CHAR(15)) # 济安金信基金评级(综合评级)
+    JAJXEARNINGPOWERRAT = Column(CHAR(15)) # 济安金信基金评级(盈利能力)
+    JAJXACHIEVESTABILITYRAT = Column(CHAR(15)) # 济安金信基金评级(业绩稳定性)
+    JAJXANTIRISKRAT = Column(CHAR(15)) # 济安金信基金评级(抗风险能力) 
+    JAJXSTOCKSELECTIONRAT = Column(CHAR(15)) # 济安金信基金评级(选股能力)
+    JAJXTIMESELECTIONRAT = Column(CHAR(15)) # 济安金信基金评级(择时能力)
+    JAJXBENCHMARKTRACKINGRAT = Column(CHAR(15)) # 济安金信基金评级(基准跟踪能力)
+    JAJXEXCESSEARNINGSRAT = Column(CHAR(15)) # 济安金信基金评级(超额收益能力)
+    JAJXTOTALFEERAT = Column(CHAR(15)) # 济安金信基金评级(整体费用)
\ No newline at end of file
