@@ -175,8 +175,9 @@ class IndexDailyCollection(Base):
     pb_mrq = Column(DOUBLE(asdecimal=False))  # 市净率-MRQ
     pe_ttm = Column(DOUBLE(asdecimal=False))  # 市盈率-MMT
     peg_ttm = Column(DOUBLE(asdecimal=False))  # PEG-MMT
-    roe_ttm = Column(DOUBLE(asdecimal=False))  # 净资产收益率-MMT
-    dy_ttm = Column(DOUBLE(asdecimal=False))  # 股息率-MMT
+    est_peg = Column(DOUBLE(asdecimal=False))  # 预测peg
+    roe = Column(DOUBLE(asdecimal=False))  # 净资产收益率-MMT
+    dy = Column(DOUBLE(asdecimal=False))  # 股息率-MMT
     pe_pct = Column(DOUBLE(asdecimal=False))  # PE百分位
     pb_pct = Column(DOUBLE(asdecimal=False))  # PB百分位
     val_score = Column(DOUBLE(asdecimal=False))  # 估值评分
@@ -249,9 +250,10 @@ class IndexDailyCollection(Base):
 
             'pb_mrq': 'PB',
             'pe_ttm': 'PE',
-            'peg_ttm': '预测PEG',
-            'roe_ttm': 'ROE',
-            'dy_ttm': '股息率',
+            'peg_ttm': 'PEG-TTM',
+            'est_peg': '预测PEG',
+            'roe': 'ROE',
+            'dy': '股息率',
             'pe_pct': 'PE百分位',
             'pb_pct': 'PB百分位',
             'val_score': '估值评分',
