@@ -12,20 +12,32 @@ from .report import ReportHelper
 from ...data.manager.manager_fund import FundDataManager
 from ...data.struct import AssetWeight, AssetPrice, AssetPosition, AssetValue
 from ...data.struct import FundPosition, TAAParam, AssetTradeParam, FundTradeParam, FAParam
+from ...util.calculator import Calculator
 
 class FundEngine:
 
     DEFAULT_CASH_VALUE = 1e8
 
-    def __init__(self, data_manager: FundDataManager, trader, taa_params:TAAParam=None, fa_params:FAParam=None, taa_param_details:dict=None):
+    def __init__(self, data_manager: FundDataManager, trader, taa_params:TAAParam=None, fa_params:FAParam=None, taa_param_details:dict=None, is_old_version=False, white_list:set=None, black_list:set=None, fund_score_funcs=None):
         self._dm = data_manager
         self._saa_helper = SAAHelper()
         self._taa_helper = TAAHelper(taa_params=taa_params, taa_param_details=taa_param_details) if taa_params or taa_param_details else None 
         self._fa_helper = FAHelper(fa_params=fa_params) if fa_params else None
         self._report_helper = ReportHelper()
         self._trader = trader
+        self._is_old_version = is_old_version
+        self._fund_score_funcs = fund_score_funcs
         self._pending_trades = []
+        # 黑白名单和全部基金列表处理 得到不允许交易基金列表， 对其中基金评分惩罚
+        self.white_list = white_list
+        self.black_list = black_list
+        self.whole_list = set(self._dm.dts.fund_list)
+        disproved_set_1 = self.whole_list - self.white_list if self.white_list else set([]) 
+        disproved_set_2 = self.black_list if self.black_list else set([])
+        self.disproved_set = disproved_set_1.union(disproved_set_2)
         assert not self.is_fund_backtest or self._fa_helper, 'if this is a fund backtest, fa_helper is necessary'
+        if self.is_fund_backtest:
+            self._trader.set_helper(self._fa_helper)
 
     def init(self):
         if not self._dm.inited:
@@ -53,6 +65,8 @@ class FundEngine:
             self._taa_helper.setup(saa)
         self._report_helper.setup(saa)
         self._trader.setup(saa)
+        if self.is_fund_backtest:
+            self._fa_helper.setup(saa)
 
     @property
     def is_fund_backtest(self):
@@ -68,29 +82,45 @@ class FundEngine:
         self._prep_asset_price = self._dm.get_index_price_data(dt)
         self._prep_fund_nav = self._dm.get_fund_nav(dt) if self.is_fund_backtest else {}
         self._prep_fund_unit_nav = self._dm.get_fund_unit_nav(dt) if self.is_fund_backtest else {}
-        self._prep_fund_score, self._prep_fund_score_raw = self._dm.get_fund_scores(dt) if self.is_fund_backtest else ({}, {})
+        self._prep_fund_score, self._prep_fund_score_raw = self._dm.get_fund_scores(dt=dt, fund_score_funcs=self._fund_score_funcs) if self.is_fund_backtest else ({}, {})
+        self._prep_fund_score, self._prep_fund_score_raw = self._dm.white_and_black_list_filter(self._prep_fund_score, self._prep_fund_score_raw, self.disproved_set)
+
         # ALLOCATION
         self._prep_target_asset_allocation = self.calc_asset_allocation(dt)
         self._prep_target_fund_allocation = self.calc_fund_allocation(dt) if self.is_fund_backtest else None
 
     def calc_trade(self, dt, cur_asset_position: AssetPosition, cur_fund_position: FundPosition=None):
-        v_asset_position, asset_trade_list = self._trader.calc_asset_trade(dt, cur_asset_position, self._prep_asset_price, self._prep_target_asset_allocation)
-        if asset_trade_list and self.is_fund_backtest:
+        if not self.is_fund_backtest:
+            v_asset_position, asset_trade_list = self._trader.calc_asset_trade(dt, cur_asset_position, self._prep_asset_price, self._prep_target_asset_allocation)
+            return asset_trade_list
+        else:
+            if self._is_old_version:
+                v_asset_position, asset_trade_list = self._trader.calc_asset_trade(dt, cur_asset_position, self._prep_asset_price, self._prep_target_asset_allocation)
+                expired_con = self._trader.has_expired_fund(cur_fund_position, self._prep_fund_score )
+                if asset_trade_list or expired_con:
+                    self._report_helper.update_rebalance_detail(dt, expired_con)
+                    v_fund_position, fund_trade_list = self._trader.calc_fund_trade(
+                        dt, self._prep_target_fund_allocation, cur_fund_position, self._prep_fund_nav, 
+                        self._prep_fund_purchase_fees, self._prep_fund_redeem_fees
+                    )
+                    return fund_trade_list
+                else:
+                    return None
             assert cur_fund_position, 'cur_fund_position should not be None in fund backtest run'
             assert self._prep_target_fund_allocation, 'target fund allocation should be prepared already'
-            v_fund_position, fund_trade_list = self._trader.calc_fund_trade(
-                dt, self._prep_target_fund_allocation, cur_fund_position, self._prep_fund_nav, 
-                self._prep_fund_purchase_fees, self._prep_fund_redeem_fees
+            v_fund_position, fund_trade_list, trigger_reason = self._trader.calc_trade(
+                dt, self._prep_target_fund_allocation, self._prep_target_asset_allocation,
+                cur_fund_position, cur_asset_position, 
+                self._prep_fund_nav, self._prep_fund_score, self._prep_asset_price
             )
+            self._report_helper.update_rebalance_detail(dt, trigger_reason)
             return fund_trade_list
-        else:
-            return asset_trade_list
 
-    def finalize_trade(self, dt, trades: list, cur_asset_position: AssetPosition, cur_fund_position: FundPosition=None):
+    def finalize_trade(self, dt, trades: list, cur_asset_position: AssetPosition, cur_fund_position: FundPosition=None, disproved_set: set={}):
         if self.is_fund_backtest:
             assert cur_fund_position, 'cur_fund_position should not be None in fund backtest run'
             self._pending_trades, traded_list = self._trader.finalize_trade(dt, trades, self._prep_asset_price, cur_asset_position, cur_fund_position, self._prep_fund_nav, self._prep_fund_unit_nav,
-                self._prep_fund_purchase_fees, self._prep_fund_redeem_fees)
+                self._prep_fund_purchase_fees, self._prep_fund_redeem_fees, disproved_set)
         else:
             self._pending_trades, traded_list = self._trader.finalize_trade(dt, trades, self._prep_asset_price, cur_asset_position)
         return traded_list 
@@ -124,6 +154,8 @@ class FundBacktestEngine(FundEngine):
         # init days
         start_date = start_date or self._dm.start_date
         end_date = end_date or self._dm.end_date
+        start_date = start_date if isinstance(start_date, datetime.date) else datetime.datetime.strptime(start_date,'%Y%m%d').date()
+        end_date = end_date if isinstance(end_date, datetime.date) else datetime.datetime.strptime(end_date,'%Y%m%d').date()
 
         # setup helpers
         self.setup(saa)
@@ -143,7 +175,7 @@ class FundBacktestEngine(FundEngine):
         self.prep_data(dt)
         _tm_prep_data = time.time()
         # finalize trade
-        traded_list = self.finalize_trade(dt, self._pending_trades, self.cur_asset_position, self.cur_fund_position)
+        traded_list = self.finalize_trade(dt, self._pending_trades, self.cur_asset_position, self.cur_fund_position, self.disproved_set)
         _tm_finalize_trade = time.time()
         # calc trade
         trade_list = self.calc_trade(dt, self.cur_asset_position, self.cur_fund_position)
@@ -266,6 +298,43 @@ def taa_result(m: FundDataManager):
                         LowPlus = 0.06) 
     return ReportHelper.get_taa_result(index_id=index_id, start_date=start_date, end_date=end_date, taaParam=taaParam, dm=m)
 
+def four_dimensions_backtest(m,begin_date, end_date, saa, fund_param, taa_detail):
+    asset_param = AssetTradeParam() 
+    t = AssetTrader(asset_param)
+    saa_bk = FundBacktestEngine(data_manager=m, trader=t, taa_params=None)
+    saa_bk.init()
+    saa_bk.run(saa=saa,start_date=begin_date,end_date=end_date)
+    saa_mv = saa_bk.get_asset_result()['market_value']
+    asset_param = AssetTradeParam() 
+    t = AssetTrader(asset_param)
+    taa_bk = FundBacktestEngine(data_manager=m, trader=t, taa_params=None,taa_param_details=taa_detail)
+    taa_bk.init()
+    taa_bk.run(saa=saa,start_date=begin_date,end_date=end_date)
+    taa_mv = taa_bk.get_asset_result()['market_value']
+    asset_param = AssetTradeParam() 
+    
+    t = FundTrader(asset_param, fund_param)
+    fa_param = FAParam() 
+    fund_taa = FundBacktestEngine(data_manager=m, trader=t, taa_params=None, fa_params=fa_param, taa_param_details=taa_detail)
+    fund_taa.init()
+    fund_taa.run(saa=saa,start_date=begin_date,end_date=end_date)
+    fund_result = fund_taa.get_fund_result()
+    fund_mv = fund_result['market_value']
+    fund_annual_ret = fund_result['annual_ret']
+    fund_mv = fund_mv.rename(columns={'mv':'优选基金'})
+    taa_mv = taa_mv.rename(columns={'mv':'战术配置'})
+    saa_mv = saa_mv.rename(columns={'mv':'战略配置'})
+    mv_df = pd.concat([fund_mv,taa_mv, saa_mv], axis =1)
+    mv_df = mv_df.join(m.dts.index_price[['hs300']].rename(columns={'hs300':'沪深300'}))
+    mv_df_ret = mv_df/mv_df.shift(1)
+    res = []
+    for port_name in mv_df:
+        res_i = Calculator.get_stat_result_from_df(df=mv_df.reset_index(), date_column='date', value_column=port_name).__dict__
+        res_i['策略名称'] = port_name
+        res.append(res_i)
+    stat_df = pd.DataFrame(res).set_index('策略名称').reindex(['沪深300','战略配置','战术配置','优选基金'])[['annualized_ret','annualized_vol','sharpe','mdd','mdd_date1','mdd_date2']]
+    return mv_df_ret, stat_df, fund_annual_ret#, mv_df
+
 def test():
     from ...data.manager.score import FundScoreManager
     m = FundDataManager('20150101', '20160101', score_manager=FundScoreManager())
