@@ -15,7 +15,7 @@ if CURRENT_PLATFORM == 'Darwin':
     mpl.rcParams['font.family'] = ['Heiti TC']
 else:
     mpl.rcParams['font.family'] = ['STKaiti']
-
+pd.plotting.register_matplotlib_converters()
 class FundPainter(object):
 
     @staticmethod
@@ -209,7 +209,7 @@ class FundPainter(object):
         name_dic = fund_info[['fund_id','desc_name']].set_index('fund_id')
         b_d = fund_nav.index[0]
         e_d = fund_nav.index[-1]
-        bench_df = index_price.loc[b_d:e_d,[asset]]
+        bench_df = index_price.loc[b_d:e_d,[asset]].dropna()
         bench_df = bench_df/bench_df.iloc[0]
         for i in range(len(date_list) - 1):   
             b_d = date_list[i]
@@ -217,6 +217,7 @@ class FundPainter(object):
             if b_d == e_d:
                 break
             bench_df_tmp = bench_df.loc[b_d:e_d,:]
+            bench_df_tmp = bench_df_tmp/bench_df_tmp.iloc[0]
             f_l = [k  for k ,v in fund_w[b_d].items() if isinstance(v, float) and round(v ,3 ) > 0]
             if 'cash' in f_l:
                 f_l.remove('cash')
@@ -224,7 +225,7 @@ class FundPainter(object):
             mv_b = bench_df.loc[b_d,asset]
             fund_tmp = fund_nav.loc[b_d:e_d,f_l].copy()
             fund_tmp = fund_tmp/fund_tmp.iloc[0]
-            fund_tmp = fund_tmp*mv_b
+            fund_tmp = fund_tmp
             res = []
             submit_d = traded_to_submit_date[b_d]
             if len(f_l) < 1:
@@ -249,10 +250,14 @@ class FundPainter(object):
             if is_tuning:
                 fund_not_select = [f for f, v in fund_score[submit_d][asset].items() if f not in f_l][:10]
                 if len(fund_not_select) < 1:
-                    break
+                    plt.legend(fontsize=17, loc = 'lower left')
+                    plt.title(f'{asset} {b_d} {e_d}', fontsize=25)
+                    plt.grid()
+                    ax.xaxis.set_ticks_position('top')
+                    continue
                 fund_tmp = fund_nav.loc[b_d:e_d,fund_not_select].copy()
                 fund_tmp = fund_tmp/fund_tmp.iloc[0]
-                fund_tmp = fund_tmp*mv_b
+                fund_tmp = fund_tmp
                 for f in fund_not_select:
                     desc = name_dic.loc[f,'desc_name']
                     f_i_dict= fund_indicator.loc[f,submit_d].to_dict()
@@ -270,7 +275,7 @@ class FundPainter(object):
 
             plt.legend(fontsize=17, loc = 'lower left')
             plt.title(f'{asset} {b_d} {e_d}', fontsize=25)
-            plt.suptitle(FundScoreManager().funcs[asset].__dict__, y=0.87, fontsize=18)
+            #plt.suptitle(FundScoreManager().funcs[asset].__dict__, y=0.87, fontsize=18)
             ax.xaxis.set_ticks_position('top')
             fund_df = pd.DataFrame(res)
             fund_df = fund_df[['desc_name','fund_id','weight','alpha','beta','track_err','fee_rate','score_raw','score']]
@@ -413,7 +418,7 @@ class FundPainter(object):
                     fund_i_fee = fee_df.loc[fund_id,'trade_fee']
                     fee_discount_l = np.linspace(1,1-fund_i_fee,index_df_i.shape[0])
                     index_df_i[fund_id] = index_df_i[fund_id] * fee_discount_l
-                w_df = df_i[['fund_id','weight']].set_index('fund_id').T
+                w_df = df_i[['fund_id','volume']].set_index('fund_id').T
                 weighted_s = (index_df_i * w_df.values).sum(axis=1)
                 ws = weighted_s.iloc[:-1]
                 ws = ws / ws[0] * last_value
@@ -424,12 +429,23 @@ class FundPainter(object):
             asset_weighted_index = index_price[[index_id]].join(asset_weighted_index).fillna(method='ffill').dropna()
             asset_weighted_index = asset_weighted_index / asset_weighted_index.iloc[0]
             asset_weighted_index.plot.line(figsize=(15,12))
+            last_value = round(asset_weighted_index[index_id+' fund weighted sum'].values[-1],4)
             plt.title(f'{index_id} funds weighted price vs {index_id} ', fontsize=25)
-            plt.suptitle('trading fee equally split on each day', y=0.87, fontsize=17)
+            plt.suptitle(f'last asset value {last_value}', y=0.87, fontsize=17)
             plt.legend(fontsize=17, loc = 'lower left')
             plt.grid()
             plt.show()
 
+    @staticmethod
+    def plot_recent_ret(df_plot_i:pd.DataFrame,begin_date:datetime.date,end_date:datetime.date,st:str,port_id:str):
+        df_plot_i.index = [str(i) for i in df_plot_i.index]
+        df_plot_i.plot.line(figsize=(12,6))
+        plt.title(f'portfolio {port_id} mv from {begin_date} to {end_date}', fontsize=20)
+        plt.suptitle(st,y=0.87,fontsize=17)
+        plt.legend(loc='lower left',fontsize = 15)
+        plt.grid()
+        plt.show()
+
     @staticmethod
     def plot_fund_rank( score_dic:pd.DataFrame, 
                         fund_info:pd.DataFrame, 
@@ -453,4 +469,94 @@ class FundPainter(object):
         score.plot.line(figsize=(12,6))
         plt.title(f'{fund_id}  {desc_name}  rank  ', fontsize=25)
         plt.grid()
-        plt.show()
\ No newline at end of file
+        plt.show()
+
+    @staticmethod
+    def plot_rolling_analysis( annual_ret:pd.DataFrame, 
+                               fund_annual_ret:float,
+                               year:int):
+        
+        df1 = annual_ret.dropna()[['沪深300','战略配置']]
+        index = df1.index
+        saa1 = df1.沪深300
+        taa1 = df1.战略配置
+        df2 = annual_ret.dropna()[['战略配置','战术配置']]
+        index = df2.index
+        saa2 = df2.战略配置
+        taa2 = df2.战术配置
+        df3 = annual_ret.dropna()[['战术配置','优选基金']]
+        index = df3.index
+        saa3 = df3.战术配置
+        taa3 = df3.优选基金
+        df4 = annual_ret.dropna()[['沪深300','优选基金']]
+        index = df4.index
+        saa4 = df4.沪深300
+        taa4 = df4.优选基金
+
+        df1.plot.line(figsize=(16,6))
+        plt.fill_between(index, saa1, taa1,
+                        where=(saa1 < taa1),
+                        alpha=0.30, color='#ff7f0e', interpolate=False)
+        plt.fill_between(index, saa1, taa1,
+                        where=(saa1 > taa1),
+                        alpha=0.30, color='#1f77b4', interpolate=False)
+        pl.legend(loc='upper left',fontsize = 17)
+        pl.title(f'滚动{year}年化收益对比', fontsize=20)
+        plt.grid()
+        plt.show()
+        
+        df2.plot.line(figsize=(16,6))
+        plt.fill_between(index, saa2, taa2,
+                        where=(saa2 < taa2),
+                        alpha=0.30, color='#ff7f0e', interpolate=False)
+        plt.fill_between(index, saa2, taa2,
+                        where=(saa2 > taa2),
+                        alpha=0.30, color='#1f77b4', interpolate=False)
+        pl.legend(loc='upper left',fontsize = 17)
+        pl.title(f'滚动{year}年化收益对比', fontsize=20)
+        plt.grid()
+        plt.show()
+        
+        df3.plot.line(figsize=(16,6))
+        plt.fill_between(index, saa3, taa3,
+                        where=(saa3 < taa3),
+                        alpha=0.30, color='#ff7f0e', interpolate=False)
+        plt.fill_between(index, saa3, taa3,
+                        where=(saa3 > taa3),
+                        alpha=0.30, color='#1f77b4', interpolate=False)
+        pl.legend(loc='upper left',fontsize = 17)
+        pl.title(f'滚动{year}年化收益对比', fontsize=20)
+        plt.grid()
+        plt.show()
+        df4.plot.line(figsize=(16,6))
+        plt.fill_between(index, saa4, taa4,
+                        where=(saa4 < taa4),
+                        alpha=0.30, color='#ff7f0e', interpolate=False)
+        plt.fill_between(index, saa4, taa4,
+                        where=(saa4 > taa4),
+                        alpha=0.30, color='#1f77b4', interpolate=False)
+        pl.legend(loc='upper left',fontsize = 17)
+        pl.title(f'滚动{year}年化收益对比', fontsize=20)
+        plt.grid()
+        plt.show()
+        df_funds = annual_ret.dropna()[['优选基金']].rename(columns={'优选基金':'value'})
+        df_funds.plot.hist(bins=20, figsize=(16,10),legend=None)
+        plt.title(f'基金滚动{year}年收益分布', fontsize=25)
+        plt.grid()
+        plt.show()
+        df_funds = df_funds.groupby('value')['value'].agg('count').pipe(pd.DataFrame).rename(columns = {'value': 'frequency'})
+        df_funds['pdf'] = df_funds['frequency'] / sum(df_funds['frequency'])
+        df_funds['cdf'] = df_funds['pdf'].cumsum()
+        df_funds.index.name = '年化收益'
+        index_loc = df_funds.index.values[df_funds.index.values > fund_annual_ret][0]
+        cdf_loc = df_funds.loc[index_loc, 'cdf']
+        df_funds = 1 - df_funds[['cdf']]
+        df_funds[['cdf']].plot.line(figsize=(12,6), legend=None)
+        plt.scatter(index_loc, 1-cdf_loc, marker='D', color='C1', s=60)
+        plt.title(f'{year}年年化收益 累积密度分布', fontsize=20)
+        rate = round((1-cdf_loc)*100,2)
+        st = f'组合年化收益 {round(fund_annual_ret * 100, 3)}%  获取年化收益胜率 {rate}%'
+        plt.suptitle(st,y=0.87,fontsize=17)
+        plt.grid()
+        plt.show()
+ 
\ No newline at end of file
