@@ -6,6 +6,7 @@ from pprint import pprint
 import platform
 import matplotlib as mpl
 import json
+from collections import Counter
 from .painter.asset_painter import AssetPainter
 from .painter.fund_painter import FundPainter
 from .painter.hybrid_painter import HybridPainter
@@ -73,6 +74,7 @@ class ReportHelper:
         self.fund_score = {}
         self.fund_score_raw = {}
         self.target_allocation = {}
+        self._rebalance_details = []
 
     def update(self,dt:datetime, asset_cur_position:AssetPosition, asset_cur_price:AssetPrice, pend_trades:list, fund_cur_position:FundPosition, fund_nav:dict, traded_list:list, fund_score:dict, fund_score_raw:dict, target_allocation):   
         # 检查回测时使用
@@ -165,7 +167,7 @@ class ReportHelper:
             d1 = hold_days[hold_days.index(d) + 1]
             idx0 = index_list.index(d)
             idx1 = index_list.index(d1)
-            result[d] = (idx1 - idx0) / 242
+            result[d.strftime('%Y-%m-%d')] = (idx1 - idx0) / 242
         w['hold_years'] = result # dic
         return w
 
@@ -278,6 +280,11 @@ class ReportHelper:
             self.index_fee = self.index_fee.groupby('index_id').sum()
             self.index_fee['fee_ratio_avg'] = self.index_fee.commission / self.index_fee.amount
 
+    def get_rebalance_detail(self):
+        rebalance_details = [i['rebalance_reason'][0]['rebalanace_reason'] for i in self._rebalance_details]
+        rebalance_details = dict(Counter(rebalance_details))
+        return rebalance_details
+
     def backtest_asset_plot(self):
         bt_type = 'asset'
         print(f'{bt_type} report')
@@ -298,19 +305,19 @@ class ReportHelper:
         pprint(w)
         w['market_value'] = mv
         self.get_index_fee_avg()
-        FundPainter.plot_fund_weights(self.fund_weights_history, self.fund_cash_history, self.fund_marktet_price_history, self.saa_weight)
-        HybridPainter.plot_market_value(self.fund_mv, bt_type, self.index_price, self.saa_weight, w)
+        #FundPainter.plot_fund_weights(self.fund_weights_history, self.fund_cash_history, self.fund_marktet_price_history, self.saa_weight)
+        HybridPainter.plot_market_value_with_mdd_analysis(self.fund_mv, bt_type, self.index_price, self.saa_weight, w)
         FundPainter.plot_fund_mdd_periods(self.fund_mv, self.fund_weights_history, self.fund_nav, self.fund_info)
         FundPainter.plot_fund_mdd_amounts(self.fund_mv, self.fund_nav, self.fund_info, self.fund_position_history.copy())
         FundPainter.plot_fund_alpha(self.fund_nav, self.fund_info, self.index_price, self.fund_position_history.copy(), w)
         FundPainter.plot_mv_on_each_asset(self.fund_nav, self.fund_info, self.index_price, self.fund_position_history.copy(), w)
     
         #HybridPainter.plot_asset_fund_mv_diff(self.asset_mv, self.fund_mv)
-        if not self.index_fee.empty:
-            FundPainter.plot_index_fund_fee(self.index_fee)
-        if not self.turnover_df.empty:
-            FundPainter.plot_fund_ret_each_year(self.turnover_df, w['mdd'], w['annual_ret'] )
-            FundPainter.plot_turnover_rate_each_year(self.turnover_df, self.turnover_rate_yearly_avg)
+        # if not self.index_fee.empty:
+        #     FundPainter.plot_index_fund_fee(self.index_fee)
+        # if not self.turnover_df.empty:
+        #     FundPainter.plot_fund_ret_each_year(self.turnover_df, w['mdd'], w['annual_ret'] )
+        #     FundPainter.plot_turnover_rate_each_year(self.turnover_df, self.turnover_rate_yearly_avg)
  
     def plot_whole(self):
         self.backtest_fund_plot()
@@ -346,9 +353,14 @@ class ReportHelper:
         self.get_fund_trade().to_csv(csv_title + '_fund_trade.csv')
         self.get_fund_position().to_csv(csv_title + '_fund_pos.csv')
         pd.DataFrame(self.asset_position_history).T.to_csv(csv_title + '_asset_pos.csv')
-        self.asset_mv.to_csv(csv_title + '_asset_mv.csv')
+        if hasattr(self, 'asset_mv'):
+            self.asset_mv.to_csv(csv_title + '_asset_mv.csv')
         self.get_asset_trade().to_csv(csv_title + '_asset_trade.csv')
     
+    def update_rebalance_detail(self, dt, rebalance_reason):
+        if bool(rebalance_reason):
+            self._rebalance_details.append({'datetime':dt,'rebalance_reason':rebalance_reason})
+
     @staticmethod
     def get_taa_result(index_id:str, start_date:datetime.date, end_date:datetime.date, taaParam:TAAParam, dm:FundDataManager):
         taa_helper = TAAHelper(taa_params=taaParam)
@@ -395,7 +407,90 @@ class ReportHelper:
                                    begin_date=begin_date,
                                    fund_index_map=dm.dts.fund_index_map)
 
+    @staticmethod
+    def indicator_score_d(begin_d:datetime.date, fund_list:list, direction:str, m:FundDataManager):
+        indicator_1 = m.dts.fund_indicator[(m.dts.fund_indicator.fund_id.isin(fund_list)) &
+                            (m.dts.fund_indicator.datetime == begin_d)]
+        indicator_1 = indicator_1[['fund_id','alpha','beta','track_err','fee_rate','index_id']].rename(columns={'alpha':f'{direction}_alpha',
+                                                                                                            'beta':f'{direction}_beta',
+                                                                                                            'track_err':f'{direction}_track_err'})
+        indicator_1 = indicator_1.set_index('fund_id')
+        score_day_dic = m._score_manager.score_cache[begin_d]
+        fund_ranks = {}
+        for index_id, score_dic in score_day_dic.items():
+            _rank = sorted(score_dic.items(), key=lambda x:x[1], reverse=True)
+            fund_rank = [i[0] for i in _rank]
+            fund_ranks[index_id] = fund_rank
+        rank_result = []
+        for r in indicator_1.itertuples():
+            try:
+                rank_result.append(fund_ranks[r.index_id].index(r.Index) + 1)   
+            except:
+                rank_result.append(10000)
+        indicator_1.loc[:,f'{direction}_rank'] = rank_result
+        return indicator_1
+    
+    @staticmethod
+    def fund_trade_result(_bk_class,_bk_result,m):
+        _dts = m.dts.trading_days.datetime.tolist()
+        rebalance_date = _bk_result['rebalance_date']
+        begin_date = rebalance_date[0]
+        mv_df = _bk_result['market_value']
+        rebalance_list = [begin_date] + [i for i in rebalance_date if i > begin_date] + [mv_df.index[-1]]
+        fund_info_dic = m.dts.fund_info[['fund_id','desc_name']].set_index('fund_id').to_dict()['desc_name']
+        df_list = []
+        for i in range(len(rebalance_list)-1):
+            begin_d = rebalance_list[i]
+            end_d = rebalance_list[i+1]
+            score_day_1 = _dts[_dts.index(begin_d) -1]
+            score_day_2 = _dts[_dts.index(end_d) -1]
+            pos_dic = _bk_class._report_helper.fund_position_history[begin_d]
+            res = []
+            for fund_id, fund_pos_i in pos_dic.items():
+                if fund_pos_i['volume'] > 1:
+                    res.append({
+                        'fund_id':fund_pos_i['fund_id'],
+                        'volume':fund_pos_i['volume'],
+                        'price1':fund_pos_i['price'],
+                        'index_id':fund_pos_i['index_id'],
+                    })
+            pos_i_df = pd.DataFrame(res)
+            pos_i_df.loc[:,'price2'] = pos_i_df['fund_id'].map(lambda x : m.dts.fund_nav.loc[end_d,x])
+            pos_i_df['amount'] = pos_i_df['volume'] * pos_i_df['price1']
+            pos_i_df['weight'] = pos_i_df['amount'] / pos_i_df['amount'].sum()
+            pos_i_df.loc[:,'desc_name'] = pos_i_df['fund_id'].map(lambda x: fund_info_dic[x])
+            pos_i_df['fund_ret_rate'] = (pos_i_df['price2'] / pos_i_df['price1']) - 1
+            pos_i_df['fund_return'] = pos_i_df['fund_ret_rate'] * pos_i_df['weight'] 
+            pos_i_df_show = pos_i_df[['fund_id','desc_name','index_id','weight','fund_ret_rate','fund_return']]
+            pos_i_df_show = pos_i_df_show.sort_values('fund_return', ascending=False)
+            fund_list = pos_i_df_show.fund_id.tolist()
+            pos_i_df_show = pos_i_df_show.set_index('fund_id')
+            indicator_1 = ReportHelper.indicator_score_d(score_day_1, fund_list, 'buy', m)
+            indicator_2 = ReportHelper.indicator_score_d(score_day_2, fund_list, 'sell', m)
+            pos_i_df_show = pos_i_df_show.join(indicator_1[['buy_alpha','buy_beta','buy_track_err','fee_rate','buy_rank']]).join(indicator_2[['sell_alpha','sell_beta','sell_track_err','sell_rank']])
+            pos_i_df_show = pos_i_df_show.reset_index()
+            pos_i_df_show.index.name = f'{begin_d}~{end_d}'
+            pos_i_df_show.loc[:,'buy_signal_date'] = begin_d
+            pos_i_df_show.loc[:,'sell_signal_date'] = end_d
+            df_list.append(pos_i_df_show)
+        return pd.concat(df_list)
 
+    @staticmethod
+    def recent_week_return(mv_df,m,benchmark,begin_date=None,port_id=''):
+        today = datetime.date.today()
+        if begin_date is None:
+            begin_date =  today - datetime.timedelta(days=today.weekday())
+        trading_days = sorted(list(set(m.dts.trading_days.datetime) & set(mv_df.index)))
+        mv_df = mv_df.loc[begin_date:].reindex(trading_days)
+        stats = Calculator.get_stat_result_from_df(df=mv_df.reset_index(), date_column='date', value_column='mv')
+        annual_ret= round(stats.annualized_ret,3)
+        mdd = round(stats.mdd,4)
+        df_plot_i = mv_df.join(m.dts.index_price[[benchmark]]).loc[begin_date:]
+        df_plot_i = df_plot_i / df_plot_i.iloc[0]
+        #df_plot_i.plot.line(figsize=(12,6))
+        total_ret = round((stats.last_unit_nav - 1) * 10000, 1)
+        st = f'check period annual ret : {annual_ret} mdd : {mdd} total_ret {total_ret} bp'
+        FundPainter.plot_recent_ret(df_plot_i,begin_date,today,st,port_id)
 
 if __name__ == "__main__":
     f._report_helper.plot_fund_score(dm=m)
